@@ -704,15 +704,6 @@ static int isinSSL(int fd)
 				return 1;
 			}
 		}
-		else
-		if( buf[0] == 22 ){ /* ConentType:handshake */
-			rcc = RecvPeek(fd,buf,sizeof(buf));
-			ERROR("SSL Hello?%d [%X %d %d %d %d]",rcc,buf[0],
-				buf[1],buf[2],buf[3]<<8|buf[4],buf[5]);
-			if( buf[5] == 1 ){
-				return 1;
-			}
-		}
 	}
 	return 0;
 }
