@@ -12,6 +12,8 @@ extern INETADDRV4 _inet_addrV4(PCStr(addr));
 #define inet_addrV4(a)	_inet_addrV4(a)
 #define inet_ntoaV4I(i)	_inet_ntoaV4I(i)
 
+int xinet_pton(int af,const char *src,void *dst);
+
 #ifdef _MSC_VER
 #include "ywinsock.h"
 #include "ysocket.h"
@@ -97,6 +99,45 @@ struct sockaddr_un {
 };
 #endif
 
+#if !defined(AF_INET6) || !defined(IPPROTO_IPV6)
+#ifndef AF_INET6
+#define AF_INET6 -6
+#endif
+struct in6_addr {
+	unsigned char addr8[16];
+};
+struct sockaddr_in6 { 
+	unsigned char   sin6_len;
+	unsigned char   sin6_family;
+	unsigned short  sin6_port;
+	unsigned int    sin6_flowinfo;
+	struct in6_addr sin6_addr;
+	unsigned int    sin6_scope_id;
+}; 
+struct addrinfo {
+	int	ai_flags;
+	int	ai_family;
+	int	ai_socktype;
+	int	ai_protocol;
+   unsigned int	ai_addrlen;
+	char	*ai_canonname;
+struct sockaddr *ai_addr;
+struct addrinfo *ai_next;
+};
+#define gethostbyname2(name,af) ((af==AF_INET)?gethostbyname(name):NULL)
+#define inet_pton(af,src,dst) -1
+#define inet_ntop(af,src,dst,siz) NULL
+#define getaddrinfo(host,serv,hint,res) -1
+#define getnameinfo(sa,salen,host,hlen,serv,slen,flags) -1
+#define freeaddrinfo(ai) -1
+#define AI_NUMERICHOST 4
+#define IPPROTO_IPV6 41
+#endif
+
+#ifndef IPV6_V6ONLY
+#define IPV6_V6ONLY -1
+#endif
+
 struct hostent *EX_GETHOSTBYNAME(PCStr(name));         /* system's standard resolver */
 struct hostent *EX_GETHOSTBYADDR(PCStr(addr),int,int); /* system's standard resolver */
 struct hostent *_GETHOSTBYNAME(PCStr(name));         /* resolvy */
@@ -181,13 +222,16 @@ struct sockaddr	_sa;
 #endif
 
 typedef struct sockaddr *SAP;
+#define IPV6_ADDRLENG	20
 
 int   VSA_afunix(VSAddr *sap,PVStr(host),int size);
 int   VSA_addrisANY(VSAddr *sap);
 int   VSA_addr(VSAddr *sap);
+int   VSA_addrX(VSAddr *sap,unsigned int ap[4]);
+int   VSA_cto_(char addr[]);
 const char *VSA_ntoa(VSAddr *sap);
 const char *VSA_htoa(struct hostent *hp);
-const char *VSA_ltoa(unsigned char *addr,int len,int type);
+const char *VSA_ltoa(const unsigned char *addr,int len,int type);
 int   VSA_port(VSAddr *sap);
 char *VSA_xtoap(VSAddr *sa,PVStr(buf),int siz);
 int   VSA_strisaddr(PCStr(addr));
@@ -200,12 +244,18 @@ int   VSA_htosa(VSAddr *sap,int port,struct hostent *hp,int hi);
 int   VSA_size(VSAddr *sap);
 int   VSA_atob(PCStr(aaddr),char baddrb[],int *btypep);
 int   VSA_decomp(VSAddr *sap,const char **baddr,int *btype,const char **bport);
+int   VSA_islocal(VSAddr *vsa);
+int   VSA_6to4(VSAddr *vsa);
+int   VSA_addrcomp(VSAddr *vsa1,VSAddr *vsa2);
+void  VSA_addrcopy(VSAddr *dst,VSAddr *src);
 void  VSA_prftp(VSAddr *sap,PVStr(mport));
 void  VSA_ftptosa(void *sap,PCStr(port));
 int   VSA_dnstosa(void *sap,int port,PCStr(revaddr));
 void  VSA_zero(VSAddr *sap);
 const char *VSA_hostlocal();
 int   isinetAddr(PCStr(saddr));
+int   sock_isv6(int sock);
+int   VSA_af(VSAddr *sap);
 
 int   SOCKS_recvfrom(int sock,char buf[],int len,int flags,SAP from,int *fromlen);
 int   SOCKS_sendto(int sock,PCStr(buf),int len,int flags,SAP to,int tolen);
