@@ -1,3 +1,5 @@
 /* __CYGWIN__ does not have errno ...
  */
+#ifndef __cplusplus
 int errno;
+#endif
