@@ -16,12 +16,12 @@ Description:
 History:
 	930923	extracted from codeconv.c of cosmos
 //////////////////////////////////////////////////////////////////////#*/
-#include <string.h>
 #include <ctype.h>
-char *malloc();
 #include <stdio.h>
-extern FILE *fopen_LIBPATH();
-extern char *getTMPDIR();
+#include <stdlib.h>
+#include "ystring.h"
+#include "file.h"
+
 int CCX_debug = 0;
 
 /*
@@ -50,7 +50,7 @@ int CCX_debug = 0;
 
 #define SJIS_DAKUTEN	0xDE
 #define SJIS_HANDAKU	0xDF
-static char *SJIS_1B_TO_JIS[] = {
+static const char *SJIS_1B_TO_JIS[] = {
 /* 0xA1-A7 */       "!#", "!V", "!W", "!\"","!%", "%r", "%!",
 /* 0xA8-AF */ "%#", "%%", "%'", "%)", "%c", "%e", "%g", "%C",
 /* 0xB0-B7 */ "!<", "%\"","%$", "%&", "%(", "%*", "%+", "%-",
@@ -60,7 +60,7 @@ static char *SJIS_1B_TO_JIS[] = {
 /* 0xD0-D7 */ "%_", "%`", "%a", "%b", "%d", "%f", "%h", "%i",
 /* 0xD8-DF */ "%j", "%k", "%l", "%m", "%o", "%s", "!+", "!,"
 };
-static char *SJIS_1B_TO_DAKUTEN[] = {
+static const char *SJIS_1B_TO_DAKUTEN[] = {
 /* 0xA1-A7 */       0,    0,    0,    0,    0,    0,    0,
 /* 0xA8-AF */ 0,    0,    0,    0,    0,    0,    0,    0,
 /* 0xB0-B7 */ 0,    0,    0,    "%t", 0,    0,    "%,", "%.",
@@ -70,7 +70,7 @@ static char *SJIS_1B_TO_DAKUTEN[] = {
 /* 0xD0-D7 */ 0,    0,    0,    0,    0,    0,    0,    0,
 /* 0xD8-DF */ 0,    0,    0,    0,    0,    0,    0,    0
 };
-static char *SJIS_1B_TO_HANDAKU[] = {
+static const char *SJIS_1B_TO_HANDAKU[] = {
 /* 0xA1-A7 */       0,    0,    0,    0,    0,    0,    0,
 /* 0xA8-AF */ 0,    0,    0,    0,    0,    0,    0,    0,
 /* 0xB0-B7 */ 0,    0,    0,    0,    0,    0,    0,    0,
@@ -92,7 +92,7 @@ static char *SJIS_1B_TO_HANDAKU[] = {
 	IS_EUC_HI(hi) && (IS_EUC_LOS(lo) || IS_EUC_LOX(lo)))
 
 
-IS_SJIS_CHAR(ch1,ch2,in_sjis)
+int IS_SJIS_CHAR(int ch1,int ch2,int in_sjis)
 {
 	if( ch1 & 0x80 ){
 		if( IS_SJIS_LO(ch2) ){
@@ -104,9 +104,9 @@ IS_SJIS_CHAR(ch1,ch2,in_sjis)
 	}
 	return 0;
 }
-IS_SJIS_STR(str)
-	unsigned char *str;
-{	unsigned char *s,ch;
+int IS_SJIS_STR(unsigned char *str)
+{	const unsigned char *s;
+	unsigned char ch;
 	int is_sjis = 0;
 
 	s = str;
@@ -120,8 +120,7 @@ IS_SJIS_STR(str)
 }
 
 unsigned char *
-SJIS_TO_JIS1(HI,LO,JCODE)
-	register unsigned char HI,LO,*JCODE;
+SJIS_TO_JIS1(unsigned char HI, unsigned char LO, unsigned char *JCODE)
 {
 	HI -= (HI <= 0x9F) ? 0x71 : 0xB1;
 	HI = (HI << 1) + 1;
@@ -136,8 +135,7 @@ SJIS_TO_JIS1(HI,LO,JCODE)
 	return JCODE;
 }
 unsigned char *
-JIS_TO_SJIS1(HI,LO,SJCODE)
-	register unsigned char HI,LO,*SJCODE;
+JIS_TO_SJIS1(unsigned char HI,unsigned char LO,unsigned char *SJCODE)
 {
 	if( HI & 1 )
 		LO += 0x1F;
@@ -153,10 +151,9 @@ JIS_TO_SJIS1(HI,LO,SJCODE)
 	return SJCODE;
 }
 
-static char *sjis_1b_to_jis(ch1,ch2,cat)
-	int *cat;
+static const char *sjis_1b_to_jis(int ch1,int ch2,int *cat)
 {	unsigned char c1,c2;
-	char *js;
+	const char *js;
 
 	c1 = (0x80 | ch1) - 0xA1;
 	c2 =  0x80 | ch2;
@@ -169,11 +166,11 @@ static char *sjis_1b_to_jis(ch1,ch2,cat)
 		return SJIS_1B_TO_JIS[c1];
 	}
 }
-static EUC_hankaku_TO_JIS(spp,dpp)
-	unsigned char **spp,**dpp;
-{	unsigned char *sp,*dp;
+static int EUC_hankaku_TO_JIS(unsigned char **spp,unsigned char **dpp,PVStr(db))
+{	const unsigned char *sp;
+	unsigned char *dp; /**/
 	unsigned char ch1,ch2;
-	char *js;
+	const char *js;
 	int cat;
 
 	sp = *spp;
@@ -189,9 +186,9 @@ static EUC_hankaku_TO_JIS(spp,dpp)
 	if( cat )
 		sp += 2;
 
-	strcpy(dp,js);
+	Xstrcpy(QVStr(dp,db),js);
 	dp += 2; *dpp = dp;
-	sp += 2; *spp = sp;
+	sp += 2; *spp = (unsigned char*)sp;
 	return 1;
 }
 
@@ -203,11 +200,11 @@ static EUC_hankaku_TO_JIS(spp,dpp)
 #define NBSP		0xA0	/* non-breaking space */
 
 
-#define sputc(sp,ch)	(sp?(*sp++ = ch):ch)
+#define sputc(dp,ch)	(dp?(*(char*)dp++ = ch):ch)
 
-static istag(str)
-	char *str;
-{	char ch,*s;
+static int istag(PCStr(str))
+{	char ch;
+	const char *s;
 
 	for( s = str; ch = *s; s++ ){
 		if( ch == '>' || isspace(ch) )
@@ -218,21 +215,22 @@ static istag(str)
 	return 0;
 }
 
-FIX_2022(src,dst,ctype)
-	char *src,*dst,*ctype;
+int FIX_2022(PCStr(src),PVStr(dst),PCStr(ctype))
 {	int in2B;
-	char ch1,ch2,*sp,*dp;
+	const char *sp;
+	char ch1,ch2;
+	refQStr(dp,dst); /**/
 	int bad;
 	int isHTML,len,ech;
 
 	in2B = 0;
 	sp = src;
-	dp = dst;
 	bad = 0;
 
 	isHTML = strcasecmp(ctype,"text/html") == 0;
 
 	while( ch1 = *sp++ ){
+		assertVStr(dst,dp+3);
 		if( ch1 == ESC ){
 			if( *sp == TO_2BCODE ){
 				if( sp[1] == 'B' || sp[1] == '@' ){
@@ -297,9 +295,8 @@ FIX_2022(src,dst,ctype)
 	return bad;
 }
 
-static is_EUC_JP(euc)
-	char *euc;
-{	char *cp;
+static int is_EUC_JP(PCStr(euc))
+{	const char *cp;
 	int ch1,ch2;
 
 	for( cp = euc; ch1 = *cp & 0xFF; cp++ ){
@@ -323,25 +320,29 @@ static is_EUC_JP(euc)
 #define CC_JIS7KANA	6
 #define CC_JIS7ROMA	7
 #define CC_UTF8		8
+#define CC_GUESS	9
+#define CC_ASIS		10
+#define CC_NONJP	11
+#define CC_EURO8	12
 #define JIS2B8(code)	(code == CC_SJIS || code == CC_EUCJP)
 #define JIS2B(code)	(code == CC_JIS2B7 || JIS2B8(code))
 
 struct {
-	char   *cs_name;
-	char   *cs_formname;
+  const	char   *cs_name;
+  const	char   *cs_formname;
 	short	cs_charcode;
 } charsets[] = {
 	{0},
 	{"jis",		"ISO-2022-JP",	CC_JIS2B7	},
-	{"euc",		"x-euc-jp",	CC_EUCJP	},
-	{"euc-jp",	"EUC-JP",	CC_EUCJP	},
-	{"sjis",	"x-sjis",	CC_SJIS		},
-	{"Shift_JIS",	"Shift_JIS",	CC_SJIS		},
+	{"x-euc-jp",	"x-euc-jp",	CC_EUCJP	},
+	{"euc",		"EUC-JP",	CC_EUCJP	},
+	{"x-sjis",	"x-sjis",	CC_SJIS		},
+	{"sjis",	"Shift_JIS",	CC_SJIS		},
 	{"utf8",	"UTF-8",	CC_UTF8		},
 	{0},
 }; 
 
-typedef struct {
+typedef struct _CCX {
 	short	cc_size;
 	short	cc_OUT; /* target charcode of conversion */
 	short	cc_out;	/* current output charcode */
@@ -349,16 +350,30 @@ typedef struct {
 	short	cc_indef; /* default input charcode */
 	short	cc_in;	/* current input charcode */
 	short	cc_previn; /* previous non-ASCII input charcode */
-	char	cc_inswitch[16]; /* detected input character type */
+	char	cc_symconv;
+	char	cc_inswitch[16]; /**//* detected input character type */
 unsigned char	cc_pinb[8];
 	int	cc_pinx;
+	int	cc_id;
 	int	cc_nonASCII;
+	short	cc_sjis;
+	short	cc_euc;
+	short	cc_nonJP;
+	short	cc_nonjp8;
+unsigned char	cc_thru8[4]; /* total size of CCX must be <= 64 */
 } CCX;
 #define cc_UTF8		cc_inswitch[CC_UTF8]
 #define cc_SJIS		cc_inswitch[CC_SJIS]
 #define cc_JIS7		cc_inswitch[CC_JIS2B7]
 #define cc_EUCJP	cc_inswitch[CC_EUCJP]
 
+#define SCtoASCII	0x04 /* multibyte alnum/symbol into to ASCII */
+#define SCtoJIS2B	0x08
+#define SCtoLF		0x10
+#define SCtoCRLF	0x20
+#define SCtoIGNBAD	0x40 /* ignore bad JIS sequence */
+#define IGN_bad(ccx)	(ccx->cc_symconv & SCtoIGNBAD)
+
 #define setccin(ccx,cc)	{ \
 	if( ccx->cc_in != ccx->cc_previn && JIS2B(ccx->cc_in) )\
 		ccx->cc_previn = ccx->cc_in; \
@@ -384,16 +399,35 @@ unsigned char	cc_pinb[8];
 #define CC_CHECKLEN	8
 #define CC_BEEUC	4
 
+void CCXcounts(CCX *ccx)
+{
+	fprintf(stderr,"UTF8[%d]",ccx->cc_UTF8);
+	fprintf(stderr,"JIS7[%d]",ccx->cc_JIS7);
+	fprintf(stderr,"SJIS[%d/%d]",ccx->cc_SJIS,ccx->cc_sjis);
+	fprintf(stderr,"EUCJP[%d/%d]\n",ccx->cc_EUCJP,ccx->cc_euc);
+}
+
 /* distinguish SJIS from EUC */
-static is_SJIS(ccx,cinb,pix,pin,pilen)
-	CCX *ccx;
-	unsigned char *cinb;
+static int guessCode(CCX *ccx, const unsigned char *cinb, int pix, int pin, int pilen)
 {	int cn,ci,ch1,ch2;
 	int cs = -1;
 	int n2K = 0,n1K = 0,n2B = 0;
 	int nJ8 = 0,nSJ = 0,nEJ = 0;
+	int nL8;
 	int as_sjis = 1;
 	int is2B;
+	int nES = 0; /* non-kanji symbols in EUC */
+	int preL8 = -1;
+	int isJP = ccx->cc_sjis + ccx->cc_euc;
+
+	nL8 = ccx->cc_nonJP; /* bias for tie break */
+	if( ccx->cc_in == CC_EURO8 && 0 < nL8 ){
+		return CC_EURO8;
+	}
+	if( 5 <= nL8 )
+		return CC_NONJP;
+	if( ccx->cc_in == CC_SJIS || ccx->cc_in == CC_EUCJP )
+		isJP = 1;
 
 	if( ccx->cc_nonASCII == 0 )
 		cn = 1024;
@@ -429,14 +463,19 @@ static is_SJIS(ccx,cinb,pix,pin,pilen)
 			if( IS_EUC(ch1,ch2) ){
 				is2B = 1;
 				nEJ++;
+				if( ch2 & 0x80 )
+					n2B++;
 				if( IS_EUC_HANKAKU(ch1,ch2) )
 					n2K++;
+				if( ch1 == 0xA3 || ch1 == 0xA4 || ch1 == 0xA5 )
+					nES++;
 			}
 		}
 		if( nJ8 == nSJ ){
 			if( IS_SJIS_2B(ch1,ch2,as_sjis) ){
 				is2B = 1;
 				nSJ++;
+				if( ch2 & 0x80 )
 				n2B++;
 			}
 			if( IS_SJIS_1B(ch1) ){
@@ -446,11 +485,29 @@ static is_SJIS(ccx,cinb,pix,pin,pilen)
 					ci--;
 			}
 		}
+		if( n2B == 0 )
+		if( isJP == 0 )
+		{
+			/* distinguish from SJIS where 8bit char. appear closely */
+			if( preL8 < 0 || 2 < (ci-preL8) ){
+				preL8 = ci;
+				nL8++;
+			}
+		}
 		nJ8++;
+
+		if( 0 < n2B )
 		if( nSJ != nEJ || nSJ < nJ8 || CC_CHECKLEN <= nJ8 )
 			break;
 	}
 	if( 0 < nJ8 ){
+		if( n2B == 0 && nJ8 <= nL8 && nSJ <= nL8 && nEJ <= nL8 ){
+/*
+ fprintf(stderr,"#### EURO8? nJ8=%d n2B=%d nSJ=%d nEJ=%d nL8=%d %d/%d\n",
+nJ8,n2B,nSJ,nEJ,nL8,ci,cn);
+*/
+			cs = CC_NONJP;
+		}else
 		if( nSJ == nEJ ){
 			if( inNonASCII(ccx,CC_SJIS) ) cs = CC_SJIS; else
 			if( inNonASCII(ccx,CC_EUCJP) ) cs = CC_EUCJP; else
@@ -459,6 +516,9 @@ static is_SJIS(ccx,cinb,pix,pin,pilen)
 				 || ccx->cc_indef == CC_SJIS )
 					cs = ccx->cc_indef;
 				else
+				if( n1K == nES )
+					cs = CC_EUCJP;
+				else
 				/* long indistinguish SJIS is not likely to be */
 				if( CC_BEEUC < nJ8 )
 					cs = CC_EUCJP;
@@ -472,50 +532,203 @@ static is_SJIS(ccx,cinb,pix,pin,pilen)
 	}
 
 	if( CCX_debug )
-	fprintf(stderr,"#### %3d %3d <%02X %02X> J%d S%d(%d) E%d(%d) [%s]\n",
+	fprintf(stderr,"#### %3d %3d <%02X %02X> J%d S%d(%d) E%d(%d)%d [%s]\n",
 		ccx->cc_nonASCII,ci,ch1,ch2,
-		nJ8, nSJ,n1K, nEJ,n2K,
+		nJ8, nSJ,n1K, nEJ,n2K,nES,
 		cs==CC_SJIS?"SJIS":(cs==CC_EUCJP?"EUC":"?"));
 
+/*
+if( ccx->cc_nonJP && (cs == CC_SJIS|cs == CC_EUCJP) )
+ fprintf(stderr,"## MISSED GUESS-A nJ8=%d n2B=%d nSJ=%d nEJ=%d nL8=%d %d\n",
+nJ8,n2B,nSJ,nEJ,nL8,ci);
+if( isJP && cs == CC_NONJP )
+ fprintf(stderr,"## MISSED GUESS-B nJ8=%d n2B=%d nSJ=%d nEJ=%d nL8=%d %d\n",
+nJ8,n2B,nSJ,nEJ,nL8,ci);
+*/
+
+	if( cs == CC_SJIS )
+		ccx->cc_sjis++;
+	if( cs == CC_EUCJP )
+		ccx->cc_euc++;
+	if( cs == CC_NONJP )
+		ccx->cc_nonJP++;
+
+	return cs;
+}
+static int is_SJIS(CCX *ccx,const unsigned char *cinb,int pix,int pin,int pilen)
+{	int cs;
+
+	cs = guessCode(ccx,cinb,pix,pin,pilen);
 	return cs == CC_SJIS;
 }
 
-CCXwithSJIS(ccx)
-	CCX *ccx;
+static int UTF8toLocal(int ch1,int ch2,int ch3,int charset,unsigned char *op,const unsigned char *ox);
+int JIS_TO_ASCII(int chset,int ch1,int ch2,int ch3)
+{	int chA;
+	unsigned char j2[3];
+
+	chA = 0;
+	if( chset == CC_UTF8 ){
+		if( UTF8toLocal(ch1,ch2,ch3,CC_EUCJP,j2,j2+1) == 2 ){
+			ch1 = j2[0];
+			ch2 = j2[1];
+			chset = CC_EUCJP;
+		}
+	}
+	if( chset == CC_EUCJP ){
+		ch1 = ch1 & 0x7F;
+		ch2 = ch2 & 0x7F;
+	}else
+	if( chset == CC_SJIS ){
+		SJIS_TO_JIS1(ch1,ch2,j2);
+		ch1 = j2[0];
+		ch2 = j2[1];
+	}
+	switch( ch1 ){
+		case '!':
+			switch( ch2 ){
+				case '!': chA = ' '; break;
+				case '$': chA = ','; break;
+				case '%': chA = '.'; break;
+				case '#': chA = '.'; break;
+				case '\'': chA = ':'; break;
+				case '(': chA = ';'; break;
+				case ')': chA = '?'; break;
+				case '-': chA = '\''; break;
+				case '.': chA = '\''; break;
+				case '*': chA = '!'; break;
+				case '0': chA = '^'; break;
+				case '1': chA = '~'; break;
+				case '2': chA = '_'; break;
+				case '=': chA = '-'; break;
+				case '?': chA = '/'; break;
+				case '@': chA = '\\'; break;
+				case 'A': chA = '~'; break;
+				case 'C': chA = '|'; break;
+				case 'F': chA = '`'; break;
+				case 'G': chA = '\''; break;
+				case 'H': chA = '"'; break;
+				case 'I': chA = '"'; break;
+				case 'J': chA = '('; break;
+				case 'K': chA = ')'; break;
+				case 'N': chA = '['; break;
+				case 'O': chA = ']'; break;
+				case 'P': chA = '{'; break;
+				case 'Q': chA = '}'; break;
+				case 'R': chA = '<'; break;
+				case 'S': chA = '>'; break;
+				case '\\': chA = '+'; break;
+				case 'Z': chA = '['; break;
+				case '[': chA = ']'; break;
+				case ']': chA = '-'; break;
+				case 'a': chA = '='; break;
+				case 'c': chA = '<'; break;
+				case 'd': chA = '>'; break;
+				case 'o': chA = '\\'; break;
+				case 'p': chA = '$'; break;
+				case 's': chA = '%'; break;
+				case 't': chA = '#'; break;
+				case 'u': chA = '&'; break;
+				case 'v': chA = '*'; break;
+				case 'w': chA = '@'; break;
+			}
+			break;
+		case '#':
+			if( isalnum(ch2) )
+				chA = ch2;
+			break;
+	}
+	return chA;
+}
+
+int CCXwithSJIS(CCX *ccx)
 {
 	return ccx->cc_SJIS;
 }
-CCXclear(ccx)
-	CCX *ccx;
+int CCXwithJP(CCX *ccx)
+{
+	return ccx->cc_SJIS | ccx->cc_EUCJP | ccx->cc_JIS7 | ccx->cc_UTF8;
+}
+void CCXthru8(CCX *ccx,PCStr(thru8))
+{	int ti,tch;
+
+	for(ti = 0; ti < sizeof(ccx->cc_thru8) && (tch = thru8[ti]); ti++)
+		ccx->cc_thru8[ti] = tch;
+	if( ti < sizeof(ccx->cc_thru8) )
+		ccx->cc_thru8[ti] = 0;
+}
+static int isthru8(CCX *ccx, unsigned char ch)
+{	int ti,tch;
+	for(ti = 0; ti < sizeof(ccx->cc_thru8) && (tch = ccx->cc_thru8[ti]); ti++ )
+		if( tch == ch )
+			return ch;
+	return 0;
+}
+void CCXclear(CCX *ccx)
 {
 	bzero(ccx,ccx->cc_size);
 }
-CCXactive(ccx)
-	CCX *ccx;
+int CCXactive(CCX *ccx)
 {
 	return ccx->cc_OUT != CC_THRU;
 }
-char *CCXcharset(ccx)
-	CCX *ccx;
+int CCXguessing(CCX *ccx)
+{
+	return ccx->cc_OUT == CC_GUESS;
+}
+int CCXoutcharset(CCX *ccx,const char **xcode)
+{	const char *code = 0;
+
+	switch( ccx->cc_OUT ){
+		case CC_ASIS:	code = ""; break;
+		case CC_ASCII:	code = "US-ASCII"; break;
+		case CC_UTF8:	code = "utf8"; break;
+		case CC_JIS2B7:	code = "iso-2022-jp"; break;
+		case CC_SJIS:	code = "Shift_JIS"; break;
+		case CC_EUCJP:	code = "EUC-JP"; break;
+		case CC_GUESS:	code = "guess"; break;
+	}
+	if( xcode )
+		*xcode = code;
+	if( code != 0 )
+		return 1 | ccx->cc_symconv;
+	else	return 0;
+}
+const char *CCXcharset(CCX *ccx)
 {
 	if( ccx->cc_outx )
 		return charsets[ccx->cc_outx].cs_formname;
 	else	return 0;
 }
-char *CCXident(ccx)
-	CCX *ccx;
+const char *CCXident(CCX *ccx)
 {
 	if( ccx->cc_UTF8 ) return "UTF-8";
+	if( ccx->cc_euc <= ccx->cc_sjis )
 	if( ccx->cc_SJIS ) return "Shift_JIS";
 	if( ccx->cc_EUCJP) return "EUC-JP";
 	if( ccx->cc_JIS7 ) return "ISO-2022-JP";
 	return "US-ASCII";
 }
-static ccx_codex(ccn,cxp)
-	char *ccn;
-	int *cxp;
+static int scanFlags(PCStr(ccn),const char **nccn)
+{	int flag = 0;
+
+	for(; *ccn; ccn += 2 ){
+		if( strneq(ccn,"a-",2) ) flag |= SCtoASCII; else
+		if( strneq(ccn,"b-",2) ) flag |= SCtoIGNBAD; else
+		if( strneq(ccn,"A-",2) ) flag |= SCtoJIS2B; else
+		if( strneq(ccn,"r+",2) ) flag |= SCtoCRLF; else
+		if( strneq(ccn,"r-",2) ) flag |= SCtoLF; else
+		break;
+	}
+	if( nccn )
+		*nccn = ccn;
+	return flag;
+}
+static int ccx_codex(PCStr(ccn),int *cxp)
 {	int cx,charcode;
-	char *name;
+	const char *name;
+
+	scanFlags(ccn,&ccn);
 
 	charcode = -1;
 	for( cx = 1; name = charsets[cx].cs_name; cx++ ){
@@ -525,9 +738,17 @@ static ccx_codex(ccn,cxp)
 			break;
 		}
 	}
+	if( charcode == -1 ){
+		if( strcaseeq(ccn,"guess") )
+			    charcode = CC_GUESS;
+	}
 	if( charcode == -1 ){
 		cx = 0;
+		if( strncaseeq(ccn,"koi8",4) ){
+		}else
 		switch( *ccn ){
+			case 0:		    charcode = CC_ASIS;   break;
+			case 'a':           charcode = CC_ASCII;  break;
 			case 'u':           charcode = CC_UTF8;   break;
 			case 'j': case 'J': charcode = CC_JIS2B7; break;
 			case 'k': case 'K': charcode = CC_JIS7K;  break;
@@ -535,26 +756,51 @@ static ccx_codex(ccn,cxp)
 				  case 'U':
 			case 'e': case 'E': charcode = CC_EUCJP;  break;
 			case 't': case 'T': charcode = CC_THRU;   break;
+			case '+':	    charcode = CC_GUESS;  break;
 			default: return -1;
 		}
 	}
 	*cxp = cx;
 	return charcode;
 }
-CCX_setincode(ccx,ccn)
-	CCX *ccx;
-	char *ccn;
+int CCXcharsetcmp(PCStr(cs1),PCStr(cs2))
+{	int Cs1,Cs2,cx;
+
+	Cs1 = ccx_codex(cs1,&cx);
+	Cs2 = ccx_codex(cs2,&cx);
+	return Cs1 - Cs2;
+}
+static char incode[64]; /**/
+void CCX_setincode(CCX *ccx,PCStr(ccn))
 {	int charcode,cx;
 
+	XStrncpy(ZVStr(incode,sizeof(incode)),ccn,sizeof(incode));
 	charcode = ccx_codex(ccn,&cx);
 	if( 0 <= charcode ){
+		if( charcode == CC_JIS2B7 ){
+			/* don't set current mode to be in 2B7 */
+		}else{
 		ccx->cc_in = charcode;
+		}
 		ccx->cc_previn = charcode;
+		ccx->cc_nonASCII += (charcode != CC_ASCII);
+	}
+	else{
+		/* not to be converted */
+		ccx->cc_nonJP = 10;
+		if( strncasecmp(ccn,"iso-8859",8) == 0
+		 || strncasecmp(ccn,"koi8",4) == 0
+		){
+			ccx->cc_in = CC_EURO8;
+			ccx->cc_previn = CC_EURO8;
+		}else{
+			ccx->cc_in = CC_NONJP;
+			ccx->cc_previn = CC_NONJP;
+		}
 	}
 }
-int CCXcreate(from,to,ccx)
-	char *from,*to;
-	CCX *ccx;
+static int CCXid;
+int CCXcreate(PCStr(from),PCStr(to),CCX *ccx)
 {	int charcode,icc;
 	int cx,icx;
 
@@ -562,12 +808,14 @@ int CCXcreate(from,to,ccx)
 	if( charcode < 0 )
 		return 0;
 
+	incode[0] = 0;
 	bzero(ccx,sizeof(CCX));
+	ccx->cc_id = ++CCXid;
+	ccx->cc_symconv = scanFlags(to,NULL);
 	ccx->cc_size = sizeof(CCX);
 	ccx->cc_OUT = charcode;
 	ccx->cc_out = CC_ASCII;
 	ccx->cc_outx = cx;
-
 	icc = ccx_codex(from,&icx);
 	if( icc < 0 )
 		icc = CC_ASCII;
@@ -578,8 +826,7 @@ int CCXcreate(from,to,ccx)
 	ccx->cc_nonASCII = 0;
 	return sizeof(CCX);
 }
-CCX *CCXnew(from,to)
-	char *from,*to;
+CCX *CCXnew(PCStr(from),PCStr(to))
 {	CCX ccxbuf,*ccx;
 	int size;
 
@@ -617,18 +864,26 @@ CCX *CCXnew(from,to)
 
 #define IS_UTF8_CONT(ch)	((ch & 0xC0) == 0x80)
 
-CCXexec(ccx,cinb,ilen,out,osiz)
-	CCX *ccx;
-	unsigned char *cinb,*out;
-{	int ch1,ch2,ch3,ch4;
-	unsigned char *pinb,*op;
+static int EUCtoUTF8(int euc,unsigned char *us,const unsigned char *ux);
+
+int CCXexec(CCX *ccx,PCStr(scinb),int ilen,PVStr(sout),int osiz)
+{	const unsigned char *cinb = (unsigned char *)scinb;
+	unsigned char *out = (unsigned char *)sout; /**/
+	int ch1,ch2,ch3,ch4;
+	int pch;
+	const unsigned char *pinb;
+	unsigned char *op; /**/
+	const unsigned char *ox;
 	int pilen,pix,pin;
 	int insjis;
-	char *js;
+	const char *js;
 	int cat;
 	int codesw;
 	int outlen;
-	unsigned char *outtop,outbuf[2048];
+	unsigned char *outtop; /**/
+	unsigned char outbuf[2048]; /**/
+	int chA;
+	int badout = 0;
 
 	if( out == cinb ){
 		if( osiz <= sizeof(outbuf) )
@@ -636,17 +891,32 @@ CCXexec(ccx,cinb,ilen,out,osiz)
 		else	outtop = (unsigned char*)malloc(osiz); 
 	}else	outtop = out;
 	op = outtop;
+	ox = outtop + (osiz - 1);
 
 	pin = ccx->cc_pinx; ccx->cc_pinx = 0;
 	pinb = ccx->cc_pinb;
 	pilen = pin + ilen;
 	pix = 0;
+	pch = -1;
+	ch1 = -1;
 
-	while( pix < pilen && (op-outtop) < osiz ){
+	while( pix < pilen && op < ox ){
+		pch = ch1;
 		ch1 = CH1;
 		pix++;
 		ch2 = CH2;
 
+		if( ccx->cc_symconv & SCtoLF ){
+			if( ch1 == '\r' && ch2 == '\n' ){
+				continue;
+			}
+		}
+		if( ccx->cc_symconv & SCtoCRLF ){
+			if( pch != '\r' && ch1 == '\n' ){
+				*op++ = '\r';
+			}
+		}
+
 		if( ch1 == 033 ){
 			/* single ESC from keyboard input must be passed
 			 * thru ...  */
@@ -689,6 +959,11 @@ CCXexec(ccx,cinb,ilen,out,osiz)
 						pix += 2;
 						continue;
 					}
+					else
+					if( ccx->cc_symconv & SCtoASCII ){
+						pix += 2;
+						continue;
+					}
 				}
 			}
 		}
@@ -726,6 +1001,12 @@ CCXexec(ccx,cinb,ilen,out,osiz)
 				break;
 			}
 			if( IS_JIS7(ch1,ch2) ){
+				if( (ccx->cc_symconv & SCtoASCII)
+				 && (chA = JIS_TO_ASCII(CC_JIS2B7,ch1,ch2,0)) )
+				{
+					toASCII(ccx,op);
+					*op++ = chA;
+				}else
 				switch( ccx->cc_OUT ){
 					case CC_SJIS:
 						JIS_TO_SJIS1(ch1,ch2,op);
@@ -736,9 +1017,10 @@ CCXexec(ccx,cinb,ilen,out,osiz)
 						*op++ = 0x80 | ch2;
 						break;
 					case CC_UTF8:
-						op += EUCtoUTF8((ch1<<8)|ch2,op);
+						op += EUCtoUTF8((ch1<<8)|ch2,op,ox);
 						break;
 					default:
+						toJIS7(ccx,op);
 						*op++ = ch1;
 						*op++ = ch2;
 						break;
@@ -791,16 +1073,22 @@ CCXexec(ccx,cinb,ilen,out,osiz)
 			}
 			if( isUTF8 ){
 			    setccin(ccx,CC_UTF8);
+
+			    if( (ccx->cc_symconv & SCtoASCII)
+			    && (chA=JIS_TO_ASCII(CC_UTF8,ch1,ch2,ch3)) ){
+				toASCII(ccx,op);
+				*op++ = chA;
+			    }else
 			    switch( ccx->cc_OUT ){
 				case CC_JIS2B7:
 				    toJIS7(ccx,op);
-				    op += UTF8toLocal(ch1,ch2,ch3,CC_JIS2B7,op);
+				    op += UTF8toLocal(ch1,ch2,ch3,CC_JIS2B7,op,ox);
 				    break;
 				case CC_EUCJP:
-				    op += UTF8toLocal(ch1,ch2,ch3,CC_EUCJP,op);
+				    op += UTF8toLocal(ch1,ch2,ch3,CC_EUCJP,op,ox);
 				    break;
 				case CC_SJIS:
-				    op += UTF8toLocal(ch1,ch2,ch3,CC_SJIS,op);
+				    op += UTF8toLocal(ch1,ch2,ch3,CC_SJIS,op,ox);
 				    break;
 				default:
 				    *op++ = ch1;
@@ -829,6 +1117,13 @@ CCXexec(ccx,cinb,ilen,out,osiz)
 			 || is_SJIS(ccx,cinb,pix-1,pin,pilen) ){
 				setccin(ccx,CC_SJIS);
 
+				if( (ccx->cc_symconv & SCtoASCII)
+				 && (chA = JIS_TO_ASCII(CC_SJIS,ch1,ch2,0)) )
+				{
+					toASCII(ccx,op);
+					*op++ = chA;
+				}
+				else
 				switch( ccx->cc_OUT ){
 				    case CC_JIS7K:
 					toJIS7K(ccx,op);
@@ -895,10 +1190,21 @@ CCXexec(ccx,cinb,ilen,out,osiz)
 
 			if( IS_EUC(ch1,ch2) )
 			if( inNonASCII(ccx,CC_EUCJP)
+/*
 			 || !is_SJIS(ccx,cinb,pix-1,pin,pilen) ){
+}
+*/
+			 || guessCode(ccx,cinb,pix-1,pin,pilen)==CC_EUCJP ){
 				setccin(ccx,CC_EUCJP);
 				pix++;
 
+				if( (ccx->cc_symconv & SCtoASCII)
+				 && (chA = JIS_TO_ASCII(CC_EUCJP,ch1,ch2,0)) )
+				{
+					toASCII(ccx,op);
+					*op++ = chA;
+				}
+				else
 				switch( ccx->cc_OUT ){
 				    case CC_JIS2B7:
 				    case CC_JIS7K:
@@ -913,7 +1219,7 @@ CCXexec(ccx,cinb,ilen,out,osiz)
 					break;
 
 				    case CC_UTF8:
-					op += EUCtoUTF8((ch1<<8)|ch2,op);
+					op += EUCtoUTF8((ch1<<8)|ch2,op,ox);
 					break;
 
 				    default:
@@ -934,6 +1240,11 @@ CCXexec(ccx,cinb,ilen,out,osiz)
 				setccin(ccx,CC_SJIS);
 				pix++;
 
+				if( (ccx->cc_symconv & SCtoASCII)
+				 && (chA = JIS_TO_ASCII(CC_SJIS,ch1,ch2,0))){
+					toASCII(ccx,op);
+					*op++ = chA;
+				}else
 				switch( ccx->cc_OUT ){
 				    case CC_JIS2B7:
 				    case CC_JIS7K:
@@ -950,7 +1261,7 @@ CCXexec(ccx,cinb,ilen,out,osiz)
 
 				    case CC_UTF8:
 					SJIS_TO_JIS1(ch1,ch2,op);
-					op += EUCtoUTF8((op[0]<<8)|op[1],op);
+					op += EUCtoUTF8((op[0]<<8)|op[1],op,ox);
 					break;
 
 				    default:
@@ -961,9 +1272,38 @@ CCXexec(ccx,cinb,ilen,out,osiz)
 				continue;
 			}
 		}
+		if( ccx->cc_in != CC_EURO8 )
 		setccin(ccx,CC_ASCII);
 		toASCII(ccx,op);
+
+		if( IGN_bad(ccx) && (ch1&0x80) ){
+			if( 0 < ccx->cc_UTF8 && (ch1&0xF0) != 0xE0 ){
+			}else{
+
+if( ccx->cc_nonJP<10 && CCXwithJP(ccx) ){
+	if( ccx->cc_nonjp8 == 0 ){
+		/*
+		fprintf(stderr,
+			"---- [%s] BadJP8 %X %o %o nJ=%d utf=%d[%X]\n",
+			incode,0xFF&ch1,0xFF&ch1,0xFF&ch2,
+			ccx->cc_nonJP,ccx->cc_UTF8,(ch1&0xF0));
+		*/
+	}
+	if( 16 < ox-op ){
+		Xsprintf(ZVStr((char*)op,ox-op),"{BadJP8:%X,%X}",0xFF&ch1,0xFF&ch2);
+		op += strlen((char*)op);
+	}
+}
+				if( badout == 0 && isthru8(ccx,ch1) ){
+					*op++ = ch1;
+				}else{
+					badout++;
+				}
+			}
+			ccx->cc_nonjp8++;
+		}else{
 		*op++ = ch1;
+		}
 	}
 	if( ilen == 0 )
 		toASCII(ccx,op);
@@ -978,11 +1318,11 @@ CCXexec(ccx,cinb,ilen,out,osiz)
 	return outlen;
 }
 
-strip_ISO2022JP(str)
-	char *str;
-{	char ch,*sp,*dp;
+void strip_ISO2022JP(PVStr(str))
+{	char ch;
+	const char *sp;
+	refQStr(dp,str); /**/
 
-	dp = str;
 	for( sp = str; ch = *sp++; ){
 		if( ch == 033 )
 		if( *sp == TO_2BCODE && (sp[1]=='@' || sp[1]=='B')
@@ -991,17 +1331,16 @@ strip_ISO2022JP(str)
 			sp += 2;
 			continue;
 		}
-		*dp++ = ch;
+		setVStrPtrInc(dp,ch);
 	}
-	*dp = 0;
+	setVStrEnd(dp,0);
 }
 
 
 int utf8n;
 int utf8err;
 
-fixUTF8(us,leng)
-	unsigned char *us;
+int fixUTF8(unsigned char *us,int leng)
 {	int cx,uc4,ucs;
 	int error;
 
@@ -1022,20 +1361,21 @@ syslog_DEBUG("{#CCX:UTF-8:ERROR:%d/%d:%x,%x#}\n",cx+1,leng,uc4,us[cx+1]);
 	}
 	return error;
 }
-fromUTF8(us,lengp,spp)
-	unsigned char *us;
-	int *lengp;
-	char **spp;
+static unsigned int fromUTF8(unsigned char *us,int *lengp,char **spp)
 {	unsigned char uc0,ucx;
 	int leng,mask,lx;
 	unsigned int uc4;
+	int len;
+	CStr(buf,64);
 
 	uc4 = 0;
 	uc0 = *us;
+	len = *lengp;
 	if( (uc0 & 0x80) == 0 ){ leng = 1; }else
 	if( (uc0 & 0x40) == 0 ){ leng = 1;
 			if( spp ){
-sprintf(*spp,"{#CCX:UTF-8:ERROR:%d/%d:%x,%x#}",1,1,uc0,us[1]);
+sprintf(buf,"{#CCX:UTF-8:ERROR:%d/%d:%x,%x#}",1,1,uc0,us[1]);
+XStrncpy(ZVStr(*spp,len),buf,len); /**/
 			utf8err++;
 			*spp += strlen(*spp);
 			uc0 = '?';
@@ -1066,9 +1406,7 @@ sprintf(*spp,"{#CCX:UTF-8:ERROR:%d/%d:%x,%x#}",1,1,uc0,us[1]);
 	*lengp = leng;
 	return uc4;
 }
-toUTF8(uc,us)
-	unsigned int uc;
-	unsigned char *us;
+int toUTF8(unsigned int uc,unsigned char *us)
 {	int tag,leng,lx;
 
 	if( uc < 0x0000080 ){ leng = 1; }else
@@ -1098,33 +1436,44 @@ unsigned short	u_u2lmap[0x10000];
 } UcsX;
 #define L2Umap	ucx->u_l2umap
 #define U2Lmap	ucx->u_u2lmap
-static UcsX *UcsXtab[8];
+static UcsX **UcsXtab;
 UcsX *UCSx(){
+	if( UcsXtab == 0 )
+		UcsXtab = (UcsX**)StructAlloc(8*sizeof(UcsX*));
 	if( UcsXtab[0] == 0 )
-		UcsXtab[0] = (UcsX*)calloc(sizeof(UcsX),1);
+		UcsXtab[0] = NewStruct(UcsX);
 	return UcsXtab[0];
 }
-static UcxInit(ucx)
-	UcsX *ucx;
+
+static int loadUnicodeMappings(PCStr(jumap),UcsX *ucx,unsigned short l2ucnt[],unsigned short u2lcnt[]);
+
+static int UcxInit(UcsX *ucx)
 {	unsigned short l2ucnt[0x10000];
 	unsigned short u2lcnt[0x10000];
+	int loaded;
 
 	if( ucx->u_init == 0 ){
-		ucx->u_init = 1;
 		bzero(l2ucnt,sizeof(l2ucnt));
 		bzero(u2lcnt,sizeof(u2lcnt));
-		loadUnicodeMappings(JUMAP,ucx,l2ucnt,u2lcnt);
+		loaded = loadUnicodeMappings(JUMAP,ucx,l2ucnt,u2lcnt);
+		if( 0 < loaded )
+			ucx->u_init = loaded;
+		else	ucx->u_init = -1;
 	}
+	return ucx->u_init;
 }
-UCSinit(){
+int UCSinit(){
 	UcsX *ucx = UCSx();
-	UcxInit(ucx);
+	return UcxInit(ucx);
+}
+void UCSreset(){
+	UcsX *ucx = UCSx();
+	ucx->u_init = 0;
 }
 
-static UTF8toLocal(ch1,ch2,ch3,charset,op)
-	unsigned char *op;
+static int UTF8toLocal(int ch1,int ch2,int ch3,int charset,unsigned char *op,const unsigned char *ox)
 {	UcsX *ucx = UCSx();
-	unsigned char buf[8];
+	CStr(buf,64);
 	int len;
 	unsigned int ucs,euc;
 
@@ -1133,7 +1482,8 @@ static UTF8toLocal(ch1,ch2,ch3,charset,op)
 	buf[1] = ch2;
 	buf[2] = ch3;
 	buf[3] = 0;
-	ucs = fromUTF8(buf,&len,NULL);
+	len = sizeof(buf);
+	ucs = fromUTF8((unsigned char*)buf,&len,NULL);
 	euc = U2Lmap[ucs];
 	if( euc  ){
 		switch( charset ){
@@ -1161,20 +1511,21 @@ static UTF8toLocal(ch1,ch2,ch3,charset,op)
 	}else{
 		switch( charset ){
 			case CC_JIS2B7:
-				sprintf(op,"\".");
+				Xsprintf(ZVStr((char*)op,ox-op),"\".");
 				break;
 			default:
-sprintf(op,  "{#CCX:UTF-8:NOMAP:UTF8toLocal:0x%04x:%x,%x,%x#}",ucs,ch1,ch2,ch3);
+sprintf(buf,"{#CCX:UTF-8:NOMAP:UTF8toLocal:0x%04x:%x,%x,%x#}",ucs,ch1,ch2,ch3);
+XStrncpy(ZVStr((char*)op,ox-op),(char*)buf,ox-op);
 				break;
 		}
-		return strlen(op);
+		return strlen((char*)op);
 	}
 }
-EUCtoUTF8(euc,us)
-	char *us;
+static int EUCtoUTF8(int euc,unsigned char *us,const unsigned char *ux)
 {	UcsX *ucx = UCSx();
 	unsigned int ucs;
 	int len;
+	CStr(buf,64);
 
 	UcxInit(ucx);
 	ucs = L2Umap[euc];
@@ -1187,21 +1538,20 @@ EUCtoUTF8(euc,us)
 		len = toUTF8(ucs,us);
 		return len;
 	}else{
-sprintf(us,  "{#CCX:UTF-8:NOMAP:LocalToUTF8:%x#}",euc);
-		return strlen(us);
+sprintf(buf,"{#CCX:UTF-8:NOMAP:LocalToUTF8:%x#}",euc);
+XStrncpy(ZVStr((char*)us,ux-us),buf,ux-us);
+		return strlen((char*)us);
 	}
 	return 0;
 }
 
-static dumpCharMapCompiled(file,ucx)
-	char *file;
-	UcsX *ucx;
+static void dumpCharMapCompiled(PCStr(file),UcsX *ucx)
 {	FILE *mfp;
 	int lch,uch,rcc;
-	unsigned char buff[0x40000],*bp;
+	CStr(buff,0x40000);
 
 	if( mfp = fopen(file,"w") ){
-		bp = buff;
+		char *bp = buff; /**/
 		for( lch = 0; lch < 0x10000; lch++ )
 		if( uch = L2Umap[lch] ){
 			*bp++ = lch >> 8; *bp++ = lch;
@@ -1213,16 +1563,15 @@ static dumpCharMapCompiled(file,ucx)
 		chmodIfShared(file,0644);
 	}
 }
-static loadCharMap(file,ucx)
-	char *file;
-	UcsX *ucx;
+static int loadCharMap(PCStr(file),UcsX *ucx)
 {	FILE *mfp;
-	char xpath[1024];
+	CStr(xpath,1024);
 	int rcc;
 	unsigned int lch,uch;
-	unsigned char buff[0x40000],*bp;
+	unsigned char buff[0x40000];
+	const unsigned char *bp;
 
-	if( mfp = fopen_LIBPATH(file,"r",xpath) ){
+	if( mfp = fopen_LIBPATH(file,"r",AVStr(xpath)) ){
 		rcc = fread(buff,1,sizeof(buff),mfp);
 		syslog_DEBUG("##CCX LOADED %s %d\n",xpath,rcc);
 		for( bp = buff; bp < &buff[rcc]; bp += 4 ){
@@ -1234,10 +1583,12 @@ static loadCharMap(file,ucx)
 		fclose(mfp);
 		return 1;
 	}
+	else{
+		syslog_DEBUG("##CCX FAILED LOADING %s\n",file);
+	}
 	return 0;
 }
-static statDump(l2umap,l2ucnt)
-	unsigned short l2umap[],l2ucnt[];
+static void statDump(unsigned short l2umap[], unsigned short l2ucnt[])
 {	int filled,unified;
 	unsigned int uc4,lc4;
 
@@ -1262,12 +1613,8 @@ syslog_ERROR("##CCX DUPLICATE: %x (%d)\n",uc4,l2ucnt[uc4]);
 	}
 syslog_ERROR("##CCX UNIFIED=%d FILLED=%d\n",unified,filled);
 }
-static loadMapping1(fp,file,ucx,l2ucnt,u2lcnt)
-	FILE *fp;
-	char *file;
-	UcsX *ucx;
-	unsigned short l2ucnt[],u2lcnt[];
-{	char line[1024];
+static int loadMapping1(FILE *fp,PCStr(file),UcsX *ucx,unsigned short l2ucnt[],unsigned short u2lcnt[])
+{	CStr(line,1024);
 	unsigned int lc1,lc2,uc1;
 
 	while( fgets(line,sizeof(line),fp) ){
@@ -1300,29 +1647,24 @@ static loadMapping1(fp,file,ucx,l2ucnt,u2lcnt)
 	syslog_ERROR("##CCX loaded: %s\n",file);
 	return 1;
 }
-static loadMap1(file,ucx,l2ucnt,u2lcnt)
-	char *file;
-	UcsX *ucx;
-	unsigned short l2ucnt[],u2lcnt[];
+static int loadMap1(PCStr(file),UcsX *ucx,unsigned short l2ucnt[],unsigned short u2lcnt[])
 {	FILE *fp;
-	char xpath[1024];
+	CStr(xpath,1024);
 
-	if( fp = fopen_LIBPATH(file,"r",xpath) ){
+	if( fp = fopen_LIBPATH(file,"r",AVStr(xpath)) ){
 		loadMapping1(fp,xpath,ucx,l2ucnt,u2lcnt);
 		fclose(fp);
 		return 1;
 	}
 	return 0;
 }
-extern char *strrpbrk();
-static loadUnicodeMappings(jumap,ucx,l2ucnt,u2lcnt)
-	UcsX *ucx;
-	char *jumap;
-	unsigned short l2ucnt[],u2lcnt[];
+static int loadUnicodeMappings(PCStr(jumap),UcsX *ucx,unsigned short l2ucnt[],unsigned short u2lcnt[])
 {	FILE *fp,*mfp;
-	char *map,*getenv();
+	const char *map;
 	int loaded = 0;
-	char bpath[1024],*tmpdir,*xp;
+	CStr(bpath,1024);
+	const char *tmpdir;
+	refQStr(xp,bpath); /**/
 
 	if( isFullpath(jumap) )
 		strcpy(bpath,jumap);
@@ -1346,7 +1688,8 @@ static loadUnicodeMappings(jumap,ucx,l2ucnt,u2lcnt)
 		loaded += loadMap1(map,ucx,l2ucnt,u2lcnt);
 
 	loaded += loadMap1(jumap,ucx,l2ucnt,u2lcnt);
-/*	loaded += loadMap1("JIS0208.TXT",ucx,l2ucnt,u2lcnt); */
+	if( loaded == 0 )
+	loaded += loadMap1("JIS0208.TXT",ucx,l2ucnt,u2lcnt);
 /*	loaded += loadMap1("JIS0201.TXT",ucx,l2ucnt,u2lcnt); */
 
 	if( loaded )
@@ -1356,13 +1699,10 @@ EXIT:
 	return loaded;
 }
 
-static scanCharMapping(in,ucx,l2ucnt,u2lcnt)
-	FILE *in;
-	UcsX *ucx;
-	unsigned short l2ucnt[],u2lcnt[];
-{	char line[1024];
-	char map1[1024];
-	char *sp;
+static void scanCharMapping(FILE *in,UcsX *ucx,unsigned short l2ucnt[],unsigned short u2lcnt[])
+{	CStr(line,1024);
+	unsigned CStr(map1,1024);
+	const char *sp;
 	unsigned int uc4,lc4;
 	int len;
 
@@ -1371,12 +1711,13 @@ static scanCharMapping(in,ucx,l2ucnt,u2lcnt)
 		if( isspace(*sp) )
 			sp++;
 		*map1 = 0;
-		sscanf(sp,"%s",map1);
+		Xsscanf(sp,"%s",AVStr(map1));
 		if( *map1 == 0 )
 			break;
 		if( isxdigit(map1[0]) && map1[4] == ':' ){
 			map1[4] = 0;
-			sscanf(map1,"%x",&lc4);
+			sscanf((char*)map1,"%x",&lc4);
+			len = sizeof(map1) - 5;
 			uc4 = fromUTF8(&map1[5],&len,NULL);
 
 if( (lc4 & 0x8080) == 0 && (lc4 & 0xFF00) != 0 ) lc4 |= 0x8080;
@@ -1392,7 +1733,7 @@ if( (lc4 & 0x8080) == 0 && (lc4 & 0xFF00) != 0 ) lc4 |= 0x8080;
 	}
 	statDump(L2Umap,l2ucnt);
 }
-dumpUTF8mapping(){
+void dumpUTF8mapping(){
 	unsigned int uc4;
 	for( uc4 = 0x001; uc4 < 0x080; uc4++ ){
 		printf("%04x:%c\r\n",uc4,uc4);
@@ -1413,7 +1754,7 @@ dumpUTF8mapping(){
 		);
 	}
 }
-dumpJIS7mapping()
+void dumpJIS7mapping()
 {	int ch,ch1,ch2,nc;
 
 	nc = 0;
@@ -1429,9 +1770,10 @@ dumpJIS7mapping()
 	}
 	printf("\r\n");
 }
-dumpEUCJPmapping()
+void dumpEUCJPmapping()
 {	int ch,ch1,ch2,nc;
 
+	nc = 0;
 	for( ch = 0x8080; ch < 0x10000; ch++ ){
 		ch1 = 0xFF & (ch >> 8);
 		ch2 = 0xFF & ch;
@@ -1444,8 +1786,7 @@ dumpEUCJPmapping()
 	}
 	printf("\r\n");
 }
-dumpCharMapping(code)
-	char *code;
+void dumpCharMapping(PCStr(code))
 {
 	switch( *code ){
 		case 'u': dumpUTF8mapping(); break;
@@ -1453,326 +1794,66 @@ dumpCharMapping(code)
 		case 'e': dumpEUCJPmapping(); break;
 	}
 }
-loadCharMapping(code,ifp)
-	char *code;
-	FILE *ifp;
+void loadCharMapping(PCStr(code),FILE *ifp)
 {	UcsX *ucx = UCSx();
 	unsigned short l2ucnt[0x10000];
 	unsigned short u2lcnt[0x10000];
 
 	bzero(l2ucnt,sizeof(l2ucnt));
 	bzero(u2lcnt,sizeof(u2lcnt));
+/*
 	scanCharMapping(ifp,ucx,L2Umap,l2ucnt,U2Lmap,u2lcnt);
+*/
+	scanCharMapping(ifp,ucx,L2Umap,l2ucnt);
 	dumpCharMapCompiled(JUMAP,ucx);
 }
 
-
-int repair_JIS = 0;
-#ifdef XXXXXXXXXXXXXXXXXX /* OBSOLETE */
-static unsigned char *repairJIStoEUC(src,dstp)
-	unsigned char *src,**dstp;
-{	unsigned char *s,*d,ch1,ch2;
-
-	d = *dstp;
-	s = src;
-	while( (ch1 = s[0]) && (ch2 = s[1]) ){
-		s += 2;
-		if( ch1 == '(' )
-		if( ch2 == 'B' || ch2 == 'J' ){
-			*dstp = d;
-			return s;
-		}
-		if( !IS_JIS7(ch1,ch2) )
-			return 0;
-
-		*d++ = 0x80 | ch1;
-		*d++ = 0x80 | ch2;
-	}
-	return 0;
-}
-int TREAT_SJIS = 1;
-unsigned char *
-SJIS_TO_EUC(src,dst)
-	unsigned char *src,*dst;
-{	register unsigned char hi,lo,*sp,*dp;
-	register int in_sjis = 0;
-
-	in_sjis = IS_SJIS_STR(src);
-	for(sp=src, dp=dst; hi = sp[0]; ){
-		lo = sp[1];
-		if( TREAT_SJIS && IS_SJIS_2B(hi,lo,in_sjis) ){
-			SJIS_TO_JIS1(hi,lo,dp);
-			dp[0] |= 0x80;
-			dp[1] |= 0x80;
-			dp += 2;
-			sp += 2;
-		}else	*dp++ = *sp++;
-	}
-	*dp = 0;
-	return dst;
-}
-unsigned char *
-EUC_TO_SJIS(src,dst)
-	unsigned char *src,*dst;
-{	register unsigned char *sp,*dp;
-
-	for(sp=src, dp=dst; *sp; ){
-		if( *sp & 0x80 ){
-			if( IS_EUC_HANKAKU(sp[0],sp[1]) ){
-				*dp++ = sp[1];
-				sp += 2;
-			}else
-			if( sp[1] && (sp[1] & 0x80) ){
-				JIS_TO_SJIS1(sp[0]&0x7F,sp[1]&0x7F,dp);
-				dp += 2; sp += 2;
-			}else	sp++;
-		}else	*dp++ = *sp++;
-	}
-	*dp = 0;
-	return dst;
-}
-TO_SJIS(any,sjis,ctype)
-	char *any,*sjis;
-	char *ctype;
-{	char *euc;
-
-	euc = malloc(strlen(any)*2+1);
-	TO_EUC(any,euc,ctype);
-	if( is_EUC_JP(euc) )
-		EUC_TO_SJIS(euc,sjis);
-	else	strcpy(sjis,any);
-	free(euc);
-}
-TO_JIS(any,jis,ctype)
-	char *any;
-	unsigned char *jis;
-	char *ctype;
-{	char *euc;
-
-	if( any[0] == 0 ){
-		jis[0] = 0;
-		return;
-	}
-
-	euc = malloc(strlen(any)*2+1);
-	TO_EUC(any,euc,ctype);
-
-	/* if NOT old-Netscape ??? DeleGate/2.1.5-DeleGate/2.1.9 */
-	if( is_EUC_JP(euc) )
-		EUC_TO_JIS(euc,jis,TO_KANJI,TO_ASCII);
-	else	strcpy(jis,any);
-	free(euc);
-}
-#define Strcpy(a,b)	(strcpy((char*)a,(char*)b),&a[strlen((char*)a)])
-unsigned char *
-EUC_TO_JIS(src,dst,toK,toA)
-	unsigned char *src,*dst;
-	char *toK,*toA;
-{	register unsigned char kana_mode = 0;
-	register unsigned char cch;
-	register unsigned char *sp = src;
-	register unsigned char *dp = dst;
-	int is_JIS = 0;
-
-	while( cch = *sp++ ){
-		if( cch & 0x80 ){
-			if( !IS_EUC(cch,*sp) ){
-				if( cch == 0xA0 && is_JIS ) /* ignore NBSP */
-					continue;
-				is_JIS++;
-				*dp++ = cch;
-				continue;
-			}
-			if(!kana_mode){
-				kana_mode = ~kana_mode;
-				dp = Strcpy(dp,toK);
-			}
-			if( IS_EUC_HANKAKU(cch,*sp) ){
-				unsigned char *sp1,*dp1;
-				sp1 = sp - 1;
-				dp1 = dp;
-				EUC_hankaku_TO_JIS(&sp1,&dp1);
-				sp = sp1;
-				dp = dp1;
-			}else
-			if( *sp & 0x80 ){
-				*dp++ = cch & ~0x80;
-				*dp++ = *sp++ & ~0x80;
-			}
-		}else{
-			if(kana_mode){
-				kana_mode = ~kana_mode;
-				dp = Strcpy(dp,toA);
-			}
-			*dp++ = cch;
-		}
-	}
-	if(kana_mode)
-		dp = Strcpy(dp,toA);
-
-	if(dp)	*dp = 0;
-	return dst;
-}
-unsigned char *
-SJIS_TO_EUC1(HI,LO,EUC)
-	unsigned char HI,LO,*EUC;
-{
-	SJIS_TO_JIS1(HI,LO,EUC);
-	EUC[0] |= 0x80;
-	EUC[1] |= 0x80;
-	return EUC;
-}
-unsigned char*
-TO_EUC(jis,euc,ctype)
-	unsigned char *jis,*euc;
-	char *ctype;
-{	register unsigned char *s,c,jis_stat;
-	register to1B,to2B;
-	register int in_sjis = 0;
-	int in_1Bkana = 0;
-	unsigned char *d;
-	static int nje;
-	int n8bits;
-	int is_JIS;
-
-	nje++;
-	n8bits = 0;
-	s = jis;
-	d = euc;
-	jis_stat = 0;
-	to2B = TO_2BCODE;
-	to1B = TO_1BCODE;
-	in_sjis = IS_SJIS_STR(jis);
-	is_JIS = 0;
-
-	while( c = *s++ ){
-		if( c == 0x80 )
-			continue; /* ignore it */
-		if( c == 0xA0 && is_JIS )
-			continue; /* ignore Non-breaking space */
-
-		if( c == to2B && jis_stat == 0 && repair_JIS ){
-			if(*s == 'B' || *s == '@'){
-				unsigned char *ts;
-				if( ts = repairJIStoEUC(s+1,&d) ){
-					s = ts;
-					continue;
-				}
-			}
-		}
-		if( c == ESC ){
-			in_1Bkana = 0;
-			if(*s == to2B){
-				if((s[1] == 'B') || (s[1] == '@')){
-					jis_stat = 0x80; s += 2;
-					is_JIS++;
-					continue;
-				}
-				jis_stat = 0;
-			}else
-			if(*s == to1B){
-				jis_stat = 0;
-				if((s[1]=='B') || (s[1]=='J') || (s[1]=='H')){
-					s += 2;
-					continue;
-				}
-				if( s[1]=='I' ){
-					s += 2;
-					is_JIS++;
-					in_1Bkana = 1;
-					continue;
-				}
-			}else
-			if(*s == ',') /* MULE */ {
-				jis_stat = 0;
-			}
-		}
-		if( is_JIS ){
-			if( c == SO ){
-				in_1Bkana = 1;
-				continue;
-			}
-			if( c == SI ){
-				in_1Bkana = 0;
-				continue;
-			}
-		}
-
-		if( c & 0x80 )
-			n8bits++;
-
-		if( !in_sjis && c == EUC_HANKAKU_HI && IS_SJIS_1B(*s) ){
-			*d++ = c;
-			*d++ = *s++;
-		}else
-		if( in_sjis&&IS_SJIS_1B(c) || in_1Bkana&&IS_SJIS_1B(0x80|c) ){
-			d[0] = EUC_HANKAKU_HI;
-			d[1] = c | 0x80;
-			d += 2;
-			is_JIS++;
-		}else
-		if( IS_SJIS_2B(c,*s,in_sjis) ){
-			SJIS_TO_EUC1(c,*s,d);
-			d += 2;
-			s++;
-			is_JIS++;
-		}else
-		if( jis_stat ){
-			if( c <= 0x20 || 0x7F <= c ){
-				*d++ = c;
-				if( c == '\n' )
-					jis_stat = 0;
-			}else{
-				if( IS_JIS7(c,*s)){
-					*d++ = jis_stat | c;
-					*d++ = jis_stat | *s++;
-				}else	*d++ = c;
-			}
-		}else{
-			if( n8bits == 0 && (c == SI || c == SO) ){
-			}else{
-				*d++ = c;
-			}
-		}
-	}
-	*d = 0;
-	return euc;
-}
-#endif
-
-CCX_TOXX(src,slen,dst,dsiz,ctype,chset)
-	unsigned char *src,*dst;
-	char *ctype,*chset;
+void CCX_TOXX(PCStr(src),int slen,PVStr(dst),int dsiz,PCStr(ctype),PCStr(chset))
 {	CCX ccx;
 
 	CCXcreate("*",chset,&ccx);
-	CCXexec(&ccx,src,slen,dst,dsiz);
+	CCXexec(&ccx,(char*)src,slen,AVStr(dst),dsiz);
 }
-CCX_TOX(src,dst,ctype,chset)
-	unsigned char *src,*dst;
-	char *ctype,*chset;
+void CCX_TOX(PCStr(src),PVStr(dst),PCStr(ctype),PCStr(chset))
 {	int len;
 
-	len = strlen(src);
-	CCX_TOXX(src,len,dst,1024+len*2,ctype,chset);
+	len = strlen((char*)src);
+	CCX_TOXX(src,len,AVStr(dst),1024+len*2,ctype,chset);
 }
-TO_EUC(any,euc,ctype)
-	char *any,*euc,*ctype;
+void TO_EUC(PCStr(any),PVStr(euc),PCStr(ctype))
 {
-	CCX_TOX(any,euc,ctype,"euc");
+	CCX_TOX(any,AVStr(euc),ctype,"euc");
 }
-TO_JIS(any,jis,ctype)
-	char *any,*jis,*ctype;
+void TO_JIS(PCStr(any),PVStr(jis),PCStr(ctype))
 {
-	CCX_TOX(any,jis,ctype,"jis");
+	CCX_TOX(any,AVStr(jis),ctype,"jis");
 }
-TO_SJIS(any,sjis,ctype)
-	char *any,*sjis,*ctype;
+void TO_SJIS(PCStr(any),PVStr(sjis),PCStr(ctype))
 {
-	CCX_TOX(any,sjis,ctype,"sjis");
+	CCX_TOX(any,AVStr(sjis),ctype,"sjis");
 }
-TO_UTF8(any,utf8,ctype)
-	char *any,*utf8,*ctype;
+void TO_UTF8(PCStr(any),PVStr(utf8),PCStr(ctype))
 {
-	CCX_TOX(any,utf8,ctype,"utf8");
+	CCX_TOX(any,AVStr(utf8),ctype,"utf8");
+}
+void TO_euc(PCStr(in),PVStr(out),int osiz)
+{	CCX ccx[1]; /**/
+
+	CCXcreate("*","euc",ccx);
+	CCXexec(ccx,(char*)in,strlen(in),AVStr(out),osiz);
+}
+void CCXfile(PCStr(icharset),PCStr(ocharset),FILE *in,FILE *out)
+{	CCX ccx[1]; /**/
+	int rcc,icc,wcc,occ;
+	CStr(buf,2*1024);
+	CStr(xbuf,8*1024);
+
+	CCXcreate(icharset,ocharset,ccx);
+	icc = occ = 0;
+	while( 0 < (rcc = fread(buf,1,sizeof(buf),in)) ){
+		icc += rcc;
+		wcc = CCXexec(ccx,buf,rcc,AVStr(xbuf),sizeof(xbuf));
+		occ += wcc;
+		fwrite(xbuf,1,wcc,out);
+	}
 }
