@@ -28,10 +28,8 @@ History:
 #include "proc.h"
 #include "log.h"
 
-#ifdef __osf__
-pid_t wait3(int *status, int options, struct rusage *rusage);
-#endif
 /*
+pid_t wait3(int *status, int options, struct rusage *rusage);
 int killpg(pid_t pgrp, int sig);
 */
 
