@@ -510,6 +510,11 @@ static int ISSOCK(int sock)
 		return 1 <= type;
 	return 0;
 }
+int Inet_pton(int af,const char *src,void *dst);
+int inet_pton(int af,const char *src,void *dst){ return Inet_pton(af,src,dst); }
+struct hostent *gethostbyname2(const char *name,int af){
+	return 0;
+}
 
 static int NO_SOCKOPEN = 1;
 /* bad if this value is zero in CHARCODE filter from FTP/HTTP on WindowsNT
@@ -706,12 +711,17 @@ int getsockHandle(int fd){ return (0 < SocketOf(fd)) ? SocketOf(fd) : -1; }
 
 int _SOCKET(int dom,int type,int proto)
 {	int sock,fd;
+	int wsaerrno;
 
 	socket_init();
 	sock = socket(dom,type,proto);
 	if( sock < 0 ){
 		if( dom == AF_UNIX )
 			LV("socket(AF_UNIX) = -1, NOT SUPPORTED");
+		wsaerrno = WSAGetLastError();
+		if( wsaerrno == WSAEAFNOSUPPORT && dom == AF_INET6 ){
+		LE("#### IPv6 is disabled or unsupported on this host ####");
+		}
 		return sock;
 	}
 	for( fd = 0; fd < MAXHANDLE; fd++ ){
