@@ -85,6 +85,7 @@ typedef struct {
 	int	re_F_aton;
   const	char   *re_confpath;
 	MStr(	re_resolv_errmsg,1024);
+	int	re_ns_stop;
 } ResolvyEnv;
 static ResolvyEnv *resolvyEnv;
 #define RE		resolvyEnv[0]
@@ -101,6 +102,7 @@ static ResolvyEnv *resolvyEnv;
 #define F_aton		RE.re_F_aton
 #define confpath	RE.re_confpath
 struct { defQStr(resolv_errmsg); } resolv_errmsg;
+#define ns_stop		RE.re_ns_stop
 
 #ifndef RSLV_CONF
 #define RSLV_CONF	"/etc/resolv.conf"
@@ -320,6 +322,9 @@ void RES_ns1(State *res,PCStr(ns),PCStr(domain))
 	CStr(nsbp,256);
 	int port;
 
+	if( ns_stop )
+		return;
+
 	clear_confid();
 	nsx = -1;
 	if( strcmp(domain,RES_NSDOM0) == 0 ){
@@ -338,7 +343,10 @@ void RES_ns1(State *res,PCStr(ns),PCStr(domain))
 		truncVStr(socks);
 		if( cp = strpbrk(ns,"# \t") )
 			truncVStr(cp);
+		/*
 		Xsscanf(socks+strlen(VIA_SOCKS),"%[-_.0-9A-Z:]",AVStr(ssb));
+		*/
+		Xsscanf(socks+strlen(VIA_SOCKS),"%[-_.0-9A-Za-z:%%]",AVStr(ssb));
 	}
 
 	port = PORT_DNS;
@@ -417,6 +425,15 @@ int RES_getns1(int nsi,VSAddr *sin)
 	return 0;
 }
 
+/* acting as a proxy */
+int RES_proxy(){
+	if( _RES.nscount == 0 )
+		return 0;
+	if( strchr(Resolvers,'D') != 0 )
+		return 1;
+	return 0;
+}
+
 int stoBV(PCStr(str),PVStr(buf),int mc,int ez){
 	const char *sp = str;
 	int sc;
@@ -517,6 +534,11 @@ static scanListFunc ns1(PCStr(name))
 {	CStr(nameb,128);
 	const char *domain;
 
+	if( strcaseeq(name,"END.") )
+		ns_stop = 1;
+	if( ns_stop )
+		return 0;
+
 	strcpy(nameb,name);
 	if( domain = strchr(nameb,'/') )
 		if( domain[1] == '/' )
@@ -534,6 +556,17 @@ int RES_ns(PCStr(nslist))
 		scan_commaList(nslist,0,scanListCall ns1);
 	return 0;
 }
+void RES_af(PCStr(af)){
+	const char *tp;
+
+	for( tp = af; *tp; tp++ ){
+		if( *tp != '4' && *tp != '6' ){
+			debug(DBG_FORCE,"unknown RES_AF[%c] %s\n",*tp,af);
+			return;
+		}
+	}
+	RES_AF = stralloc(af);
+}
 void RES_verify(PCStr(verify))
 {
 	RES_VERIFY = stralloc(verify);
@@ -912,7 +945,10 @@ static void putAddrs(struct hostent *hp)
 			break;
 		if( 0 < hi )
 		printf(",");
+		/*
 		printf("%d.%d.%d.%d",op[0],op[1],op[2],op[3]);
+		*/
+		printf("%s",VSA_ltoa(op,hp->h_length,hp->h_addrtype));
 	}
 }
 
