@@ -30,11 +30,12 @@ void sort_ipaddrs(const char *addrs[]);
 char *RES_resolvers();
 #define resolvers RES_resolvers()
 
+int ipv6_domain(PCStr(caddr),PVStr(name));
 int gethostbynameaddr_dns(PCStr(name),int qtype,int rrc,char *rrv[],PVStr(rrb),PVStr(cname));
 int gethostbynameaddr_sys(PCStr(name),int qtype,int rrc,char *rv[],PVStr(rb),PVStr(cname));
 int gethostbynameaddr_dnsrch(PCStr(name),int qtype,int rrc,char *rv[],PVStr(rb),PVStr(cname));
 int gethostbynameaddr_cache(PCStr(dir),PCStr(name),int rrc,char *rv[],PVStr(rb),int byname,PVStr(cname),int noexpire);
-void puthost_cache(PCStr(nameaddr),char *rv[],int byname,PVStr(cname));
+void puthost_cache(PCStr(nameaddr),char *rv[],int byname,PVStr(cname),int len,int type);
 int rem_unknown(char *rv[],PCStr(unknown),int leng);
 int gethostbyname_all(PCStr(where),char rwhere[],PCStr(name),int rrc,char *rv[],PVStr(rb),PVStr(cname));
 int gethostbynameaddr_file(PCStr(path),PCStr(name),int rrc,char *rv[],PVStr(rb),int byname,PVStr(cname));
@@ -94,6 +95,10 @@ int gethostbynameaddr_sys(PCStr(name),int qtype,int rrc,char *rv[],xPVStr(rb),PV
 
 	if( qtype == TY_A )
 		ht = EX_GETHOSTBYNAME(name);
+	else
+	if( qtype == TY_AAAA ){
+		ht = gethostbyname2(name,AF_INET6);
+	}
 	else{
 		VSAddr sab;
 		const char *baddr;
@@ -105,14 +110,23 @@ int gethostbynameaddr_sys(PCStr(name),int qtype,int rrc,char *rv[],xPVStr(rb),PV
 	if( ht == NULL )
 		return 0;
 
+	/*
 	if( qtype == TY_A ){
+	*/
+	if( qtype == TY_A || qtype == TY_AAAA ){
 		for( ac = 0; ap = ht->h_addr_list[ac]; ac++ ){
 			if( rrc-1 <= ac ){
 				break;
 			}
+			/*
 			Bcopy(ap,rb,4);
+			*/
+			Bcopy(ap,rb,ht->h_length);
 			rv[ac] = (char*)rb;
+			rb += ht->h_length;
+			/*
 			rb += 4;
+			*/
 		}
 	}else{
 		ac = 0;
@@ -231,11 +245,28 @@ int RES_next_res(PCStr(where),int ri,PVStr(res),PVStr(arg))
 		strcpy(arg,res+2);
 	return ri;
 }
+
+int RES_NOINET6 = 0; /* don't retry AAAA for A */
+int RES_QTYPE = 0; /* can be AF_INET6 */
+
 static int caching(int lastres,int ac,PCStr(nameaddr),char *rv[],int byname,PVStr(cname),PCStr(unknown_mark),int marklen)
 {
 	/* don't cache the result from the cache */
 	if( lastres != RT_CACHE ){
+		int len,type;
+		/*
 		puthost_cache(nameaddr,rv,byname,AVStr(cname));
+		*/
+		if( RES_QTYPE == AF_INET6 ){
+			len = 16;
+			type = AF_INET6;
+		}else{
+			len = 4;
+			type = AF_INET;
+		}
+		if( byname && RES_QTYPE == AF_INET6 )
+			byname = AF_INET6;
+		puthost_cache(nameaddr,rv,byname,AVStr(cname),len,type);
 	}else{
 		ac = rem_unknown(rv,unknown_mark,marklen);
 	}
@@ -270,9 +301,18 @@ int gethostbyname_all(PCStr(where),char rwhere[],PCStr(name),int rrc,char *rv[],
 	  lastres = res;
 	  switch(res){
 	    case RT_CACHE:
+	    {	int byname;
 		with_cache = 1;
+		if( RES_QTYPE == AF_INET6 )
+			byname = AF_INET6;
+		else	byname = 1;
+		ac = gethostbynameaddr_cache(arg,name,rrc,rv,AVStr(rb),byname,AVStr(cname),0);
+
+/*
 		ac = gethostbynameaddr_cache(arg,name,rrc,rv,AVStr(rb),1,AVStr(cname),0);
+*/
 		break;
+	    }
 	    case RT_FILE:
 		ac = gethostbynameaddr_file(arg,name,rrc,rv,AVStr(rb),1,AVStr(cname));
 		break;
@@ -280,9 +320,15 @@ int gethostbyname_all(PCStr(where),char rwhere[],PCStr(name),int rrc,char *rv[],
 		ac = gethostbynameaddr_nis(_NISMAP_NAME,arg,name,rrc,rv,AVStr(rb),1,AVStr(cname));
 		break;
 	    case RT_DNS:
+		if( RES_QTYPE == AF_INET6 ){
+ac = gethostbynameaddr_dnsrch(name,TY_AAAA,rrc,rv,AVStr(rb),AVStr(cname));
+		}else
 		ac = gethostbynameaddr_dnsrch(name,TY_A,rrc,rv,AVStr(rb),AVStr(cname));
 		break;
 	    case RT_SYS:
+		if( RES_QTYPE == AF_INET6 ){
+ac = gethostbynameaddr_sys(name,TY_AAAA,rrc,rv,AVStr(rb),AVStr(cname));
+		}else
 		ac = gethostbynameaddr_sys(name,TY_A,rrc,rv,AVStr(rb),AVStr(cname));
 		break;
 	  }
@@ -376,8 +422,12 @@ int gethostbyaddr_all(PCStr(where),char rwhere[],PCStr(caddr),int rrc,char *rv[]
 		{
 		int a1,a2,a3,a4;
 		CStr(name,256);
+		if( VSA_strisaddr(caddr) == AF_INET6 ){
+			ipv6_domain(caddr,AVStr(name));
+		}else{
 		sscanf(caddr,"%d.%d.%d.%d",&a1,&a2,&a3,&a4);
 		sprintf(name,"%d.%d.%d.%d.%s.",a4,a3,a2,a1,REVERSE_DOM);
+		}
 		if( res == RT_DNS )
 			ac = gethostbynameaddr_dns(name,TY_PTR,rrc,rv,AVStr(rb),VStrNULL);
 		else	ac = gethostbynameaddr_sys(caddr,TY_PTR,rrc,rv,AVStr(rb),VStrNULL);
@@ -408,13 +458,16 @@ int gethostbyaddr_all(PCStr(where),char rwhere[],PCStr(caddr),int rrc,char *rv[]
 	return ac;
 }
 
+char *RES_AF = "46";
 struct hostent *RES_gethostbyaddr(PCStr(baddr), int len, int type);
 struct hostent *RES_gethostbyname(PCStr(name))
 {	int ac;
 	CStr(caddr,64);
 	CStr(cname,512);
 	CStr(where,8);
-	octet *dp;
+	const char *types;
+	const char *tp;
+	int qtype;
 
 	RES_init();
 	debug(DBG_ANY,"gethostbyname(%s)\n",name);
@@ -426,7 +479,37 @@ struct hostent *RES_gethostbyname(PCStr(name))
 		return RES_gethostbyaddr(baddr,bleng,btype);
 	}
 
-	ac = gethostbyname_all(resolvers,where,name,elnumof(Raddrv),Raddrv,AVStr(Raddrb),AVStr(cname));
+	if( strcasestr(name,"IN-ADDR.ARPA") || strcasestr(name,"IP6.INT") ){
+		debug(DBG_FORCE,"don't search getbyhostname(%s)\n",name);
+		return NULL;
+	}
+
+	types = RES_AF;
+	if( strncmp(name,"-AAAA.",6) == 0 ){ types = "6"; name += 6; }else
+	if( strncmp(name,"_4.", 3) == 0 ){ types = "4";  name += 3; }else
+	if( strncmp(name,"_6.", 3) == 0 ){ types = "6";  name += 3; }else
+	if( strncmp(name,"_64.",4) == 0 ){ types = "64"; name += 4; }else
+	if( strncmp(name,"_46.",4) == 0 ){ types = "46"; name += 4; }
+
+	ac = 0;
+	for( tp = types; *tp; tp++ ){
+		if( *tp == '6' )
+			RES_QTYPE = AF_INET6;
+		else	RES_QTYPE = 0;
+		ac = gethostbyname_all(resolvers,where,name,elnumof(Raddrv),Raddrv,AVStr(Raddrb),AVStr(cname));
+
+		if( 0 < ac )
+			break;
+		if( *name == '_' ) /* retrieved _service not A/AAAA */
+			break;
+		if( tp[1] ){
+			debug(DBG_FORCE,"retrying %s [%s]\n",
+				tp[1]=='6'?"AAAA":"A",name);
+		}
+	}
+	qtype = RES_QTYPE;
+	RES_QTYPE = 0;
+
 	if( ac <= 0 )
 		return NULL;
 
@@ -440,10 +523,16 @@ struct hostent *RES_gethostbyname(PCStr(name))
 		Rnamev[0] = Rnameb; strcpy(Rnameb,name);
 		Rnamev[1] = 0;
 	}
+	if( qtype == AF_INET6 ){
+		Rhostent.h_ent.h_addrtype = AF_INET6;
+		Rhostent.h_ent.h_length = 16;
+	}else{
 	if( 1 < ac )
 		sort_ipaddrs((const char**)Raddrv);
 
 	Rhostent.h_ent.h_addrtype = AF_INET;
+	Rhostent.h_ent.h_length = 4;
+	}
 	return &Rhostent.h_ent;
 }
 
@@ -456,6 +545,9 @@ struct hostent *RES_gethostbyaddr(PCStr(baddr), int len, int type)
 	const unsigned char *ba = (const unsigned char *)baddr;
 
 	RES_init();
+	if( type == AF_INET6 ){
+		strcpy(caddr,VSA_ltoa((unsigned char*)baddr,len,type));
+	}else
 	sprintf(caddr,"%d.%d.%d.%d",ba[0],ba[1],ba[2],ba[3]);
 	debug(DBG_ANY,"gethostbyaddr(%s)\n",caddr);
 
@@ -463,6 +555,14 @@ struct hostent *RES_gethostbyaddr(PCStr(baddr), int len, int type)
 	if( ac <= 0 )
 		return NULL;
 
+	Rhostent.h_ent.h_length = len;
+/*
+	int ai;
+	for( ai = 0; ai < ac; ai++ ){
+		Rhostent.h_ent.h_addr_list[ai] = len;
+	}
+*/
+
 	Raddrv[0] = Raddrb;
 	bcopy(baddr,Raddrb,len);
 	Raddrv[1] = 0;
