@@ -48,7 +48,6 @@ char P_DBFILE[]   = "DBFILE";
 char P_DGOPTS[]   = "DGOPTS";
 char P_DGPATH[]   = "DGPATH";
 char P_DGROOT[]   = "DGROOT";
-char P_DGSIGN[]   = "DGSIGN";
 char P_DELAY[]    = "DELAY";
 char P_DELEGATE[] = "DELEGATE";
 char P_DEST[]     = "DEST";
@@ -78,6 +77,7 @@ char P_HTTPCONF[] = "HTTPCONF";
 char P_ICP[]      = "ICP";
 char P_ICPCONF[]  = "ICPCONF";
 char P_INETD[]    = "INETD";
+char P_IPV6[]     = "IPV6";
 char P_INPARAM[]  = "INPARAM";
 char P_LIBDIR[]   = "LIBDIR";
 char P_LIBPATH[]  = "LIBPATH";
@@ -110,6 +110,7 @@ char P_RPORT[]    = "RPORT";
 char P_REACHABLE[]= "REACHABLE";
 char P_REJECT[]   = "REJECT";
 char P_RELIABLE[] = "RELIABLE";
+char P_RES_AF[]   = "RES_AF";
 char P_RES_CONF[] = "RES_CONF";
 char P_RES_DEBUG[]= "RES_DEBUG";
 char P_RES_LOG[]  = "RES_LOG";
@@ -206,7 +207,6 @@ static ParamSpec params[] = {
 { P_DGOPTS,	STD,M0,1,1},
 { P_DGPATH,	STD,M0,1,1},
 { P_DGROOT,	STD,M0,1,1},
-{ P_DGSIGN,	STD,M0,1,1},
 { P_DELAY,	STD,M0,0,1, 0,scan_DELAY},
 { P_DELEGATE,	STD,M0,0,1, 0,scan_DELEGATE},
 { P_DEST,	EXP,M0,1,1},
@@ -236,6 +236,7 @@ static ParamSpec params[] = {
 { P_ICP,	STD,M0,0,0},
 { P_ICPCONF,	STD,M0,0,0},
 { P_INETD,	EXP,M0,0,0},
+{ P_IPV6,	EXP,M0,0,0},
 { P_INPARAM,	EXP,M0,0,0},
 { P_LIBDIR,	EXP,M0,1,1},
 { P_LIBPATH,	EXP,M0,1,1},
@@ -269,6 +270,7 @@ static ParamSpec params[] = {
 { P_REJECT,	DIS,M1,0,0},
 { P_RELIABLE,	DIS,M1,0,0},
 { P_REMITTABLE,	STD,M0,0,0},
+{ P_RES_AF,	STD,M0,1,1},
 { P_RES_CONF,	STD,M0,1,1},
 { P_RES_DEBUG,	STD,M0,1,1},
 { P_RES_LOG,	STD,M0,1,1},
