@@ -427,6 +427,7 @@ static int isNotServPort(PCStr(host),int port)
 	for( sx = 0; sx < SVPortN; sx++ )
 	{
 		if( SvPort(sx) == port ){
+			/* wildcard as 0.0.0.0 or :: must be ignored */
 			if( *SvHostStr(sx) ){
 				if( hostcmp_incache(SvHost(sx),host) != 0 ){
 					continue;
@@ -514,8 +515,13 @@ int (*CheckClosed)(int) = _checkCloseOnTimeout;
 /*
  *
  */
+int addrIsWildcard(PCStr(host));
 void sethostnameIF(PCStr(host))
 {
+	if( addrIsWildcard(host) ){
+		syslog_ERROR("[%s] is not set as interface addr.\n",host);
+		return;
+	}
 	strcpy(serverHost1,host);
 
 	if( serverHostPort != NULL )
@@ -604,7 +610,10 @@ int forcedIF_HPX(Connection *Conn,PCStr(vhostport),PVStr(hostport))
 }
 
 int VA_HostPortIFclnt(Connection *Conn,int clsock,PVStr(name),PVStr(addr),VAddr *Vaddr)
+/*
 {	CStr(addrb,32);
+*/
+{	CStr(addrb,64);
 
 	if( !AddrEQ(Conn->cl_sockHOST,AddrZero) )
 		goto EXIT;
@@ -640,6 +649,16 @@ static int _clientIF(Connection *Conn,PVStr(hostport),xPVStr(host),int clsock,in
 		if( byaddr && addr[0] != 0 )
 			Xstrcpy(SVStr(host,hostb) host,addr);
 	}
+{
+	char *hp;
+	if( hp = strchr(host,'%') ){
+		/* it is unnecssary to be shown to peer (client), but
+		 * it might be necessary in some case.
+		 */
+		syslog_ERROR("[%s] scope-id ignored\n",host);
+		*hp = 0;
+	}
+}
 	if( hostport != NULL )
 		sprintf(hostport,"%s:%d",host,port);
 
@@ -702,7 +721,30 @@ static int isme(PCStr(myhost),int myport,PCStr(rhost),int rport)
 	return IsMyself(rhost);
 }
 
+int IsmyselfX(Connection *Conn,PCStr(rproto),PCStr(rhost),int rport);
+static struct {
+	MStr(c_proto,32);
+	MStr(c_name,128);
+	int c_port;
+	int c_isme;
+} IsmeCache;
 int Ismyself(Connection *Conn,PCStr(rproto),PCStr(rhost),int rport)
+{	int yes;
+
+	if( IsmeCache.c_port == rport
+	 && strcaseeq(IsmeCache.c_proto,rproto)
+	 && strcaseeq(IsmeCache.c_name,rhost) ){
+		goto EXIT;
+	}
+	yes = IsmyselfX(Conn,rproto,rhost,rport);
+	strcpy(IsmeCache.c_proto,rproto);
+	strcpy(IsmeCache.c_name,rhost);
+	IsmeCache.c_port = rport;
+	IsmeCache.c_isme = yes;
+EXIT:
+	return IsmeCache.c_isme;
+}
+int IsmyselfX(Connection *Conn,PCStr(rproto),PCStr(rhost),int rport)
 {	CStr(myhost,256);
 	int myport;
 
@@ -716,6 +758,14 @@ int Ismyself(Connection *Conn,PCStr(rproto),PCStr(rhost),int rport)
 	if( isMYSELF(rhost) )
 		return 1;
 
+	/* 9.0.0
+	 * the interface at which current connection is accepted is me.
+	 * this is very likely to be true when acting as an origin server.
+	 */
+	if( CLIF_PORT == rport && hostcmp(CLIF_HOST,rhost) == 0 ){
+		return 1;
+	}
+
 	/*
 	 * "sock.dir.af-local" for AF_UNIX "/dir/sock" with port number 0xFFFF
 	 *  means that port number must not be cared
