@@ -13,11 +13,10 @@ OR IMPLIED WARRANTIES.
 #include <stdio.h>
 #include "ystring.h"
 #include "dglib.h"
-#include "log.h"
 
 #define NAME	"DeleGate"
-#define VERSION	"8.11.4"
-#define DATE	"May 20, 2005"
+#define VERSION	"9.0.0"
+#define DATE	"April 22, 2005"
 #define AUTHOR	"Yutaka Sato"
 #define A_ORG	"National Institute of Advanced Industrial Science and Technology"
 #define A_EMAIL	"ysato AT delegate DOT org"
@@ -28,45 +27,6 @@ AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165\r\n\
 Copyright (c) 1994-2000 Yutaka Sato and ETL,AIST,MITI\r\n\
 Copyright (c) 2001-2005 National Institute of Advanced Industrial Science and Technology (AIST)"
 
-static char _VERSION[32] = VERSION;
-static char _DATE[32] = DATE;
-static char DGSIGN_default[] = "VRPYMD";
-/* DGSIGN="V.R.P/Y.M.D" */
-void scan_DGSIGN(DGC*Conn,const char *mysig){
-	if( mysig ){
-		char vf[8];
-		char vs[6][32];
-		CStr(ver,32);
-		CStr(date,32);
-		int i;
-		for( i = 0; i < 6; i++ )
-			vf[i] = 'x';
-		ver[0] = date[0] = 0;
-		Xsscanf(mysig,"%[^/]/%s",AVStr(ver),AVStr(date));
-		sscanf(ver,"%c.%c.%c",&vf[0],&vf[1],&vf[2]);
-		sscanf(date,"%c.%c.%c",&vf[3],&vf[4],&vf[5]);
-		Xsscanf(VERSION,"%[^.].%[^.].%s",
-			FVStr(vs[0]),FVStr(vs[1]),FVStr(vs[2]));
-		Xsscanf(DATE,"%s %[^,], %s",
-			FVStr(vs[4]),FVStr(vs[5]),FVStr(vs[3]));
-		for( i = 0; i < 6; i++ ){
-			if( strchr("-0xX",vf[i]) ){
-				vs[i][0] = vf[i];
-				vs[i][1] = 0;
-			}
-		}
-		Xsprintf(FVStr(_VERSION),"%s.%s.%s",vs[0],vs[1],vs[2]);
-		Xsprintf(FVStr(_DATE),"%s %s, %s",vs[4],vs[5],vs[3]);
-		InitLog("DGSIGN VERSION: %s\n",_VERSION);
-		InitLog("DGSIGN DATE: %s\n",_DATE);
-	}
-}
-#undef VERSION
-#undef DATE
-#define VERSION	_VERSION
-#define DATE	_DATE
-
-
 static const char *copyright;
 const char *DELEGATE_copyright(){
 	CStr(buf,0x2000);
