@@ -19,21 +19,17 @@ History:
 	950622	created
 //////////////////////////////////////////////////////////////////////#*/
 #include <stdio.h>
-#include "ystring.h"
-#include "vsignal.h"
-extern char *DELEGATE_ver();
+#include "teleport.h"
 
 #define TP_INVITE	"INVITE"
 #define TP_ROUTE	"ROUTE"
 #define IMSIZE	(1024*8)
 
-char *MY_QZ = "3";
+const char *MY_QZ = "3";
 
 int FromTeleport;
 int TELEPORT_TIMEOUT = 5;
 int TELEPORT_BUFFERED = 1;
-static int NOWAIT;
-static int NOFLUSH;
 
 #define TPD_ERR	0
 #define TPD_DBG	1
@@ -47,7 +43,7 @@ int TELEPORT_loglev = TPD_DBG;
 #define VRB	(!LOG(TPD_VRB)) ? 0 : sv1vlog
 
 typedef struct {
-	char   *a_path;
+  const	char   *a_path;
 	int	a_io[2];
 	int	a_onEOF;
 	int	a_onCB;
@@ -61,14 +57,53 @@ typedef struct {
 	int	a_origin;
 	int	a_sentQZ;
 } Agent;
-static Agent *agents[128];
-static int nagents;
-static int nalive;
-static int agentID;
-static char teleHost[512];
-static int telePort;
+typedef struct {
+	Agent  *ve_agents[128];
+	int	ve_nagents;
+	int	ve_nalive;
+	int	ve_agentID;
+	MStr(	ve_teleHost,512);
+	int	ve_telePort;
 
-static clearZomb(kill)
+	int	ve_NOWAIT;
+	int	ve_NOFLUSH;
+
+	int	ve_PPVehicle; /* Point to Point Vehicle without Teleport */
+	int	ve_dontTryTeleport;
+	int	ve_TeleportPorts[64];
+	int	ve_dontRetryConnect;
+	jmp_buf	ve_jmpenv;
+	Agent  *ve_writingAgent;
+
+	int	ve_pcount;
+	MStr(	ve_ppath,256);
+} VehicleEnv;
+static VehicleEnv *vehicleEnv;
+#define agents		vehicleEnv->ve_agents
+#define nagents		vehicleEnv->ve_nagents
+#define nalive		vehicleEnv->ve_nalive
+#define agentID		vehicleEnv->ve_agentID
+#define teleHost	vehicleEnv->ve_teleHost
+/**/
+#define telePort	vehicleEnv->ve_telePort
+#define NOWAIT		vehicleEnv->ve_NOWAIT
+#define NOFLUSH		vehicleEnv->ve_NOFLUSH
+#define PPVehicle	vehicleEnv->ve_PPVehicle
+#define dontTryTeleport	vehicleEnv->ve_dontTryTeleport
+#define TeleportPorts	vehicleEnv->ve_TeleportPorts
+#define dontRetryConnect vehicleEnv->ve_dontRetryConnect
+#define jmpenv		vehicleEnv->ve_jmpenv
+#define writingAgent	vehicleEnv->ve_writingAgent
+#define pcount		vehicleEnv->ve_pcount
+#define Ppath		vehicleEnv->ve_ppath
+/**/
+static void minit_vehicle()
+{
+	if( vehicleEnv == 0 )
+		vehicleEnv = NewStruct(VehicleEnv);
+}
+
+static void clearZomb(int kill)
 {	int ax,aj,deleted;
 	Agent *ap;
 
@@ -80,7 +115,7 @@ static clearZomb(kill)
 			DBG("{T}.DONE agent[%d] %di+%do\n",ap->a_id,ap->a_rcc,ap->a_wcc);
 			fclose(ap->a_toa);
 			fclose(ap->a_froma);
-			free(ap->a_path);
+			free((char*)ap->a_path);
 			free(ap);
 			deleted++;
 		}else{
@@ -89,7 +124,7 @@ static clearZomb(kill)
 	}
 	nagents -= deleted;
 }
-static closeTeleportSides(){
+static void closeTeleportSides(){
 	int ax;
 	Agent *ap;
 
@@ -124,11 +159,12 @@ static closeTeleportSides(){
  *           teleport backbone process(es)
  *
  */
-static Agent *newAgent(path,svsock,telesocks,clsock)
-	char *path;
-	int telesocks[];
+static void callbackDelegate(int ifd,int cbsock,PCStr(telehost),int teleport);
+static int connectMyself(PCStr(telehost),int teleport);
+static void callAgent(int telesock,int clsock);
+static Agent *newAgent(PCStr(path),int svsock,int telesocks[],int clsock)
 {	Agent *ap;
-	char newpath[256];
+	CStr(newpath,256);
 	register int pid;
 	int cbsock;
 
@@ -144,7 +180,7 @@ static Agent *newAgent(path,svsock,telesocks,clsock)
 		sprintf(newpath,"%d!%s",ap->a_id,path);
 	}
 	DBG("{T}.SET(1) a_path=[%s]\n",newpath);
-	ap->a_path = strdup(newpath);
+	ap->a_path = stralloc(newpath);
 
 	ap->a_onEOF = -1;
 	ap->a_onCB = -1;
@@ -167,7 +203,7 @@ ap->a_id,0,ap->a_io[1],clsock,ap->a_path);
 	Socketpair(ap->a_io);
 	cbsock = -1;
 	if( clsock < 0 && NOFLUSH )
-		cbsock = server_open("callbackDeleGate",NULL,0,1);
+		cbsock = server_open("callbackDeleGate",VStrNULL,0,1);
 
 	if( (pid = Fork("newAgent")) == 0 ){
 		close(ap->a_io[1]);
@@ -205,8 +241,7 @@ ap->a_id,0,ap->a_io[1],clsock,ap->a_path);
 	return ap;
 }
 
-static replacesock(ap,clsock)
-	Agent *ap;
+static void replacesock(Agent *ap,int clsock)
 {
 	fclose(ap->a_froma);
 	fclose(ap->a_toa);
@@ -214,8 +249,7 @@ static replacesock(ap,clsock)
 	ap->a_froma = fdopen(clsock,"r");
 	ap->a_toa = fdopen(clsock,"w");
 }
-static onCallBack(ap)
-	Agent *ap;
+static int onCallBack(Agent *ap)
 {	int cbsock,clsock;
 
 	if( 0 <= (cbsock = ap->a_onCB) ){
@@ -236,8 +270,7 @@ static onCallBack(ap)
 	}
 	return 0;
 }
-static onEOF(ap)
-	Agent *ap;
+static int onEOF(Agent *ap)
 {	int clsock;
 
 	if( 0 <= (clsock = ap->a_onEOF) ){
@@ -249,71 +282,70 @@ static onEOF(ap)
 	return 0;
 }
 
-static pathReverse(path,rpath)
-	char *path,*rpath;
-{	char buf[256],*bp,ch;
-	char *rp;
-	char p1[256],p2[256];
+static void pathReverse(PCStr(path),xPVStr(rpath))
+{	CStr(buf,256);
+	const char *bp;
+	char ch;
+	refQStr(rp,rpath);
+	CStr(p1,256);
+	CStr(p2,256);
 
 	if( path[0] == '=' ){
-		*rpath++ = *path++;
+		setVStrPtrInc(rpath,*path++);
 		if( path[0] == '<' ){
-			*rpath++ = '>';
+			setVStrPtrInc(rpath,'>');
 			path++;
 		}else
 		if( path[0] == '>' ){
-			*rpath++ = '<';
+			setVStrPtrInc(rpath,'<');
 			path++;
 		}
 	}
 
 	strcpy(buf,path);
 	bp = &buf[strlen(buf)-1];
-	rp = rpath;
+	cpyQStr(rp,rpath);
 
 	while( buf < bp ){
 		ch = *bp;
 		if( ch == '!' || ch == '<' || ch == '>' ){
-			if( sscanf(bp+1,"%[^/]/%s",p1,p2) == 2 )
+			if( Xsscanf(bp+1,"%[^/]/%s",AVStr(p1),AVStr(p2)) == 2 )
 				sprintf(rp,"%s/%s",p2,p1);
 			else	strcpy(rp,bp+1);
 			rp += strlen(rp);
 			if( ch == '<' ) ch = '>';
 			if( ch == '>' ) ch = '<';
-			*rp++ = ch;
-			*bp = 0;
+			setVStrPtrInc(rp,ch);
+			truncVStr(bp);
 		}
 		bp--;
 	}
 	strcpy(rp,bp);
 	/*VRB("PATH-REVERSE: [%s]->[%s]\n",path,rpath);*/
 }
-static Agent *findAgent(path,svsock,telesocks,new)
-	char *path;
-	int telesocks[];
-	int *new;
+static Agent *findAgent(xPVStr(path),int svsock,int telesocks[],int *xnew)
 {	int ax;
 	int aid;
 	Agent *ap;
-	char *pp;
+	const char *pp;
 	char direct;
-int unbound;
-unbound = (path[0] == '=') && (path[1] == '<') && (path[2] == '!');
+	int unbound;
+	unbound = (path[0] == '=') && (path[1] == '<') && (path[2] == '!');
 
-if( *path == '=' )
-	path++;
+	if( *path == '=' )
+		path++;
 
-if( *path == '<' ){
-	direct = '<';
-	*path++;
-}else
-if( *path == '>' ){
-	char rpath[256];
-	direct = '>';
-	*path++;
-	pathReverse(path,rpath);
-	path = rpath;
-}
+	if( *path == '<' ){
+		direct = '<';
+		*path++;
+	}else
+	if( *path == '>' ){
+		CStr(rpath,256);
+		direct = '>';
+		*path++;
+		pathReverse(path,AVStr(rpath));
+		setPStr(path,rpath,sizeof(rpath));
+	}
 
 	aid = atoi(path);
 	if( aid != 0 ){
@@ -322,10 +354,10 @@ if( *path == '>' ){
 			if( ap->a_id == aid ){
 				for( pp = path; *pp; pp++ )
 					if( *pp == '<' || *pp == '>' )
-						*pp = '!';
+						*(char*)pp = '!';
 				if( ap->a_path[0] == 0 ){
 DBG("{T}.SET(2) a_path=[%s]\n",path);
-					ap->a_path = strdup(path);
+					ap->a_path = stralloc(path);
 				}
 				return ap;
 			}
@@ -333,66 +365,59 @@ DBG("{T}.SET(2) a_path=[%s]\n",path);
 		return NULL;
 	}
 
-if( *path == '!' || *path == '>' || *path == '<' ) path++;
-if( *path == '>' || *path == '<' ) path++;
-for( pp = path; *pp; pp++ ) if( *pp == '<' ) *pp = '!';
+	if( *path == '!' || *path == '>' || *path == '<' ) path++;
+	if( *path == '>' || *path == '<' ) path++;
+	for( pp = path; *pp; pp++ ) if( *pp == '<' ) *(char*)pp = '!';
 
 	for( ax = 0; ax < nagents; ax++ ){
 		ap = agents[ax];
 
-if( unbound ){
-	if( pp = strchr(ap->a_path,'!') )
-	if( streq(path,pp+1) ){
+		if( unbound ){
+			if( pp = strchr(ap->a_path,'!') )
+			if( streq(path,pp+1) ){
 		DBG("{T}.UNBOUND #%d [%s][%s]\n",ap->a_id,path,ap->a_path);
-		*new = 0;
-		return ap;
+				*xnew = 0;
+				return ap;
+			}
+		}else
+			if( streq(path,ap->a_path) ){
+				*xnew = 0;
+				return ap;
+			}
 	}
-}else
-		if( streq(path,ap->a_path) ){
-			*new = 0;
-			return ap;
-		}
-	}
-	*new = 1;
+	*xnew = 1;
 	return newAgent(path,svsock,telesocks,-1);
 }
 
-static deWrite(fd,buf,len)
-	char *buf;
+static int deWrite(int fd,PCStr(buf),int len)
 {	int olen;
-	char obuf[0x4000];
+	CStr(obuf,0x4000);
 
-	olen = QZdecode(fd,obuf,buf,len);
+	olen = QZdecode(fd,AVStr(obuf),buf,len);
 	if( olen < 0 )
 		return Write(fd,buf,len);
 	else	return Write(fd,obuf,olen);
 }
-static enRead(fd,prefix,buf,size)
-	char *prefix,*buf;
+static int enRead(int fd,PCStr(prefix),PVStr(buf),int size)
 {	int rcc;
-	char ibuf[IMSIZE];
+	CStr(ibuf,IMSIZE);
 
 	rcc = read(fd,ibuf,sizeof(ibuf));
 	if( rcc <= 0 )
 		return rcc;
-	return QZencode(fd,prefix,buf,ibuf,rcc);
+	return QZencode(fd,prefix,AVStr(buf),ibuf,rcc);
 }
-static char *fromAgent(prefix,buf,size,fp)
-	char *prefix,*buf;
-	FILE *fp;
+static char *fromAgent(PCStr(prefix),PVStr(buf),int size,FILE *fp)
 {
-	if( enRead(fileno(fp),prefix,buf,size) <= 0 )
+	if( enRead(fileno(fp),prefix,AVStr(buf),size) <= 0 )
 		return NULL;
-	else	return buf;
+	else	return (char*)buf;
 }
-static toAgent(fp,buf,len,olenp)
-	FILE *fp;
-	char *buf;
-	int *olenp;
+static int toAgent(FILE *fp,PCStr(buf),int len,int *olenp)
 {	int olen;
-	char obuf[0x4000];
+	CStr(obuf,0x4000);
 
-	*olenp = olen = QZdecode(fileno(fp),obuf,buf,len);
+	*olenp = olen = QZdecode(fileno(fp),AVStr(obuf),buf,len);
 	if( olen <= 0 ){
 		DBG("toAgent(%x,%d -> %d)\n",fp,len,olen);
 		return olen;
@@ -403,10 +428,9 @@ static toAgent(fp,buf,len,olenp)
 */
 }
 
-static invite1(host,wfp)
-	char *host;
-	FILE *wfp;
-{	char msg[1024];
+static int Fflush(FILE *fp);
+static scanListFunc invite1(PCStr(host),FILE *wfp)
+{	CStr(msg,1024);
 
 	sprintf(msg,"%s WhereIs/%s \r\n",TP_INVITE,host);
 	fputs(msg,wfp);
@@ -415,14 +439,7 @@ static invite1(host,wfp)
 	return 0;
 }
 
-static int PPVehicle;	/* Point to Point Vehicle without Teleport */
-static dontTryTeleport = 0;
-static int TeleportPorts[64];
-static int dontRetryConnect;
-static jmp_buf jmpenv;
-
-static Agent *writingAgent;
-static void sigPIPE(sig)
+static void sigPIPE(int sig)
 {
 	signal(SIGPIPE,sigPIPE);
 	if( writingAgent )
@@ -432,15 +449,19 @@ static void sigPIPE(sig)
 		longjmp(jmpenv,sig);
 	}
 }
-static resettty()
+static void resettty()
 {
-	system("stty echo");
+	if( fork() == 0 )
+		execlp("stty","stty","echo",NULL);
+	else	wait(0);
 }
-static settty()
+static void settty()
 {
 	if( isatty(0) ){
-		system("stty -echo");
-		addBeforeExit("STTY-ECHO",resettty);
+		if( fork() == 0 )
+			execlp("stty","stty","-echo",NULL);
+		else	wait(0);
+		addBeforeExit("STTY-ECHO",(vFUNCP)resettty,0);
 	}
 }
 
@@ -453,49 +474,39 @@ static settty()
  *	this process is server side of a private Teleport connection.
  *
  */
-static connectTeleport(clsocks,host,port,tunnel,telesocks)
-	int clsocks[];
-	char *host;
-	char *tunnel;
-	int telesocks[];
+static int connectTeleport(int clsocks[],PCStr(host),int port,PCStr(tunnel),int telesocks[])
 {	int fd;
 	int tio[2];
-	char ttype[64],tscript[1024];
-	char *av[32];
+	CStr(ttype,64);
+	CStr(tscript,1024);
 	int clsock;
-	char helo[128],resp[256];
+	CStr(helo,128);
+	CStr(resp,256);
 
 	if( tunnel ){
 		Socketpair(tio);
-		sscanf(tunnel,"%[^:]:%[^\r\n]",ttype,tscript);
+		fieldScan(tunnel,ttype,tscript);
 		if( Fork("Tunnel") == 0 ){
 			close(tio[1]);
 			dup2(tio[0],0);
 			dup2(tio[0],1);
 
 			shiobar(tscript);
-/*
-			av[0] = "sh";
-			av[1] = "-c";
-			av[2] = tscript;
-			av[3] = 0;
-			Execvp("Tunnel","sh",av);
-*/
-			exit(-1);
+			exit(0);
 		}
 		close(tio[0]);
 		fd = tio[1];
 		MY_QZ = "1";
 		dontRetryConnect = 1;
-{
+ {
 FILE *fp;
-char msg0[256];
+CStr(msg0,256);
 fp = fdopen(dup(fd),"r");
 setbuf(fp,NULL);
 if( fgets(msg0,sizeof(msg0),fp) != NULL )
 DBG("Remote Peer Says: %s",msg0);
 fclose(fp);
-}
+ }
 		telesocks[0] = tio[1];
 		telesocks[1] = tio[1];
 		return 0;
@@ -541,32 +552,33 @@ fclose(fp);
 	telesocks[1] = clsock;
 	return 0;
 }
-static tracemsg(who,msg)
-	char *who,*msg;
-{	char msghead[42],*np,ch;
+static void tracemsg(PCStr(who),PCStr(msg))
+{	CStr(msghead,42);
+	const char *np;
+	char ch;
 
 	if( TELEPORT_loglev < TPD_VRB )
 		return;
 
-	strncpy(msghead,msg,sizeof(msghead)-2);
-	msghead[sizeof(msghead)-2] = 0;
+	strncpy(msghead,msg,sizeof(msghead)-2); setVStrEnd(msghead,sizeof(msghead)-2);
 	if( np = strpbrk(msghead,"\r\n") )
-		*np = 0;
+		truncVStr(np);
 	for( np = msghead; ch = *np; np++ )
 		if( ch < ' ' )
-			*np = '?';
+			*(char*)np = '?';
 	VRB("{T}.%s %s\n",who,msghead);
 }
-static Fflush(fp)
-	FILE *fp;
+static int Fflush(FILE *fp)
 {
 	VRB("## FLUSH(%d)\n",fileno(fp));
 	return fflush(fp);
 }
-bindTeleportVehicle(tx,clsocks,host,port,tunnel,invites)
-	int clsocks[];
-	char *host,*tunnel,*invites;
-{	char msg[0x4000],path[0x4000],head[0x4000],*bodyp,*body;
+int bindTeleportVehicle(int tx,int clsocks[],PCStr(host),int port,PCStr(tunnel),PCStr(invites))
+{	CStr(msg,0x4000);
+	CStr(path,0x4000);
+	CStr(head,0x4000);
+	const char *bodyp;
+	const char *body;
 	register int pid;
 	FILE *svfp,*rfp,*wfp;
 	FILE *fpv[64];
@@ -574,15 +586,16 @@ bindTeleportVehicle(tx,clsocks,host,port,tunnel,invites)
 	int rdv[64];
 	int nready;
 	int ax;
-	int new;
+	int xnew;
 	Agent *apv[64],*ap;
 	int neof;
 	int retry,wsecs,wtotal;
 	int telesocks[2];
-	char resp[256];
+	CStr(resp,256);
 	int svsock,clsock;
 
-	svsock = server_open("teleport",NULL,0,50);
+	minit_vehicle();
+	svsock = server_open("teleport",VStrNULL,0,50);
 	if( svsock < 0 ){
 		ERR("{T}.TeleportVehicle can't open server socket.\n");
 		return 0;
@@ -633,7 +646,7 @@ bindTeleportVehicle(tx,clsocks,host,port,tunnel,invites)
 			wsecs = 60;
 		sleep(wsecs);
 	}
-	telePort = getpeerNAME(telesocks[0],teleHost);
+	telePort = getpeerNAME(telesocks[0],AVStr(teleHost));
 	DBG("{T}.Teleport: %s:%d [%s:%d] opened[%d/%d]\n",host,port,
 		teleHost,telePort, telesocks[0],telesocks[1]);
 
@@ -641,7 +654,7 @@ bindTeleportVehicle(tx,clsocks,host,port,tunnel,invites)
 	wfp = fdopen(telesocks[1],"w");
 
 	if( invites && invites[0] )
-		scan_commaList(invites,0,invite1,wfp);
+		scan_commaList(invites,0,scanListCall invite1,wfp);
 
 	for(;;){
 		fpc = 0;
@@ -691,7 +704,8 @@ bindTeleportVehicle(tx,clsocks,host,port,tunnel,invites)
 		for( fpx = cfpc; fpx < fpc; fpx++ ){
 		    ap = apv[fpx];
 		    if( 0 < rdv[fpx] ){
-			char path[1024],xpath[1024];
+			CStr(path,1024);
+			CStr(xpath,1024);
 
 			if( ap->a_path[0] == 0 ){
 				sprintf(path,"=<!%d ",ap->a_id);
@@ -702,11 +716,11 @@ bindTeleportVehicle(tx,clsocks,host,port,tunnel,invites)
 
 				if( ap->a_sentQZ == 0 ){
 					ap->a_sentQZ = 1;
-					sprintf(xpath+strlen(xpath),"=Q%s",MY_QZ);
+					Xsprintf(TVStr(xpath),"=Q%s",MY_QZ);
 					/* this Qn may be repeated in the first block ... */
 				}
 			}
-			if( fromAgent(xpath,msg,sizeof(msg),fpv[fpx]) == NULL ){
+			if( fromAgent(xpath,AVStr(msg),sizeof(msg),fpv[fpx]) == NULL ){
 				if( onEOF(ap) || onCallBack(ap) )
 					continue;
 
@@ -752,23 +766,23 @@ DBG("{T}.GOT HARD-EOF local, INFORM remote of it: %s\n",path);
 
 /*
 		if( msg[0] == '=' )
-			strcpy(msg,msg+1);
+			ovstrcpy(msg,msg+1);
 		else{
 			ERR("{T}.Teleport ERROR:[%d] %s\n",strlen(msg),msg);
 		}
 */
-if( msg[0] != '=' )
-{
-fprintf(stderr,"DeleGate/Teleport: ?????? %s",msg);
-sleep(1);
-continue;
-}
+ if( msg[0] != '=' )
+ {
+ fprintf(stderr,"DeleGate/Teleport: ?????? %s",msg);
+ sleep(1);
+ continue;
+ }
 		bodyp = wordScan(msg,path);
 		body = wordScan(bodyp,head);
 
-		ap = findAgent(path,svsock,telesocks,&new);
+		ap = findAgent(AVStr(path),svsock,telesocks,&xnew);
 		if( ap == NULL ){
-			char rpath[256];
+			CStr(rpath,256);
 
 /*
 			if( head[0]=='=' && head[1]=='=' ){
@@ -780,15 +794,12 @@ continue;
 */
 
 			{
-				static int pcount;
-				static char ppath[256];
-
-				if( strcmp(path,ppath) == 0 ){
+				if( strcmp(path,Ppath) == 0 ){
 					if( ++pcount % 10 == 0 )
 					ERR("NO SUCH AGENT: [%s] (%d)\n",path,pcount);
 				}else{
 					ERR("NO SUCH AGENT: [%s]\n",path);
-					strcpy(ppath,path);
+					strcpy(Ppath,path);
 					pcount = 1;
 				}
 			}
@@ -842,12 +853,11 @@ if( strchr(msg,'\n') == 0 ){
 	exit(0);
 }
 
-openTeleport(host,port,closefd)
-	char *host;
+static int openTeleport(PCStr(host),int port,int closefd)
 {	int telesock;
 	int io[2];
 	int rcc,rfdv[2];
-	char buf[0x4000];
+	CStr(buf,0x4000);
 	FILE *fpv[2];
 	FILE *wfp;
 	int qzput = 0;
@@ -882,7 +892,7 @@ openTeleport(host,port,closefd)
 				deWrite(io[0],buf,rcc);
 			}else
 			if( 0 < rfdv[1] ){
-				rcc = enRead(io[0],NULL,buf,sizeof(buf));
+				rcc = enRead(io[0],NULL,AVStr(buf),sizeof(buf));
 				if( rcc <= 0 ){ 
 					DBG("SEND SOFT-EOF to remote.\n");
 					Write(telesock,"==\r\n",4);
@@ -907,19 +917,20 @@ openTeleport(host,port,closefd)
 }
 
 #define NOWAIT_EXT "/nowait"
-teleportOpen(mx,master,mport,target_server,closefd)
-	char *master,*target_server;
-{	char *hp;
+static int bindVehicle(int telesock,PCStr(server));
+int teleportOpen(int mx,PCStr(master),int mport,PCStr(target_server),int closefd)
+{	const char *hp;
 	int tsock,rcc;
-	char msg[1024];
-	char localhost[128];
+	CStr(msg,1024);
+	CStr(localhost,128);
 
+	minit_vehicle();
 	if( dontTryTeleport )
 		return -1;
 
 	if( 0 <= TeleportPorts[mx] ){
 		strcpy(localhost,"localhost");
-		GetHostname(localhost,sizeof(localhost));
+		GetHostname(AVStr(localhost),sizeof(localhost));
 		tsock = client_open("teleport","teleport",localhost,
 			TeleportPorts[mx]);
 
@@ -931,7 +942,7 @@ teleportOpen(mx,master,mport,target_server,closefd)
 			Write(tsock,msg,strlen(msg));
 			rcc = read(tsock,msg,sizeof(msg));
 			if( 0 < rcc ){
-				msg[rcc] = 0;
+				setVStrEnd(msg,rcc);
 				DBG("{T}.Teleport[%d]: got: %s",tsock,msg);
 				return tsock;
 			}
@@ -949,8 +960,10 @@ teleportOpen(mx,master,mport,target_server,closefd)
 	}
 	return -1;
 }
-callAgent(telesock,clsock)
-{	char msg[1024],server[256],path[256];
+static void callAgent(int telesock,int clsock)
+{	CStr(msg,1024);
+	CStr(server,256);
+	CStr(path,256);
 	int sdv[2][2];
 	int cnt[2];
 	int rcc;
@@ -994,7 +1007,7 @@ if( !PPVehicle )
 	DBG("{T}.callAgent: END RELAY\n");
 */
 /*
-	sscanf(msg,"path=%s",path);
+	sscanf (msg,"path=%s",path);
 	fprintf(ts,"ROUTE %s \r\n",path);
 	Fflush(ts);
 	if( fgets(msg,sizeof(msg),fs) == NULL )
@@ -1021,23 +1034,23 @@ if( !PPVehicle )
  *  (Teleport backbone will be blocked during until these steps finish)
  */
 
-static connectMyself(telehost,teleport)
-	char *telehost;
+static int connectMyself(PCStr(telehost),int teleport)
 {	int clsock,cbsock,cbport;
-	char myhost[128],cbmsg[128],ackmsg[128];
-	int myport;
+	CStr(myhost,128);
+	CStr(cbmsg,128);
+	CStr(ackmsg,128);
 	int wcc;
 	FILE *afp;
 
-	myport = myhostport(myhost,sizeof(myhost));
-	clsock = client_open("callbackDelegate1","delegate",myhost,myport);
+	clsock = connectToMyself("callbackDelegate1");
 
 	if( 0 <= clsock ){
+		getpeerName(clsock,AVStr(myhost),"%H:%P");
 		sprintf(cbmsg,"CPORT %s:%d\r\n",telehost,teleport);
 		write(clsock,cbmsg,strlen(cbmsg));
 		if( PollIn(clsock,1*1000) <= 0 ){
-			sv1log("{T}#### No resp. from callbackDeleGate %s:%d\n",
-				myhost,myport);
+			sv1log("{T}#### No resp. from callbackDeleGate %s\n",
+				myhost);
 			close(clsock);
 			return -1;
 		}
@@ -1067,18 +1080,20 @@ static connectMyself(telehost,teleport)
 	return -1;
 }
 
-callbackDelegate(ifd,cbsock,telehost,teleport) /* maybe unnecessary */
-	char *telehost;
+static void callbackDelegate(int ifd,int cbsock,PCStr(telehost),int teleport) /* maybe unnecessary */
 {	int clsock;
-	char host[256];
+	CStr(host,256);
 	int port;
-	char imsg[4096],*imp;
+	CStr(imsg,4096);
+	refQStr(imp,imsg);
 	FILE *ifp;
 	FILE *ts;
 
 	DBG("{T}.callbackDeleGate: NOWAIT=%d NOFLUSH=%d\n",NOWAIT,NOFLUSH);
-	if( cbsock < 0 || !NOFLUSH )
-		return callDelegate1(ifd,NULL,telehost,teleport);
+	if( cbsock < 0 || !NOFLUSH ){
+		callDelegate1(ifd,NULL,telehost,teleport);
+		return;
+	}
 
 /*
 	port = gethostNAME(cbsock,host);
@@ -1104,15 +1119,20 @@ callbackDelegate(ifd,cbsock,telehost,teleport) /* maybe unnecessary */
 		}
 	}
 */
+ifp = NULL;
+host[0] = 0;
+port = 0;
 
 	clsock = client_open("callbackDelegate","teleport",host,port);
+if(ifp)
 	fclose(ifp); /* invoke ACCEPT() */
 	callDelegate1(clsock,imsg,telehost,teleport);
 }
 
-bindVehicle(telesock,server)
-	char *server;
-{	char msg[1024],mpath[1024],*mp;
+static int bindVehicle(int telesock,PCStr(server))
+{	CStr(msg,1024);
+	CStr(mpath,1024);
+	const char *mp;
 	int rcc;
 
 	sprintf(msg,"WhereIs/%s \r\n",server,MY_QZ);
@@ -1127,7 +1147,7 @@ bindVehicle(telesock,server)
 	if( rcc <= 0 )
 		return -1;
 
-	msg[rcc] = 0;
+	setVStrEnd(msg,rcc);
 	DBG("[%d] GOT RESPONSE: %s",telesock,msg);
 
 	if( msg[0] == '=' && msg[1] == '>' ){
@@ -1135,7 +1155,7 @@ bindVehicle(telesock,server)
 		mpath[0] = '<';
 		for( mp = mpath+1; *mp; mp++ )
 			if( *mp == '>' )
-				*mp = '!';
+				*(char*)mp = '!';
 		sprintf(msg,"%s %s \r\n",TP_ROUTE,mpath);
 		Write(telesock,msg,strlen(msg));
 
@@ -1144,7 +1164,7 @@ bindVehicle(telesock,server)
 			return -1;
 
 		rcc = RecvLine(telesock,msg,sizeof(msg));
-		msg[rcc] = 0;
+		setVStrEnd(msg,rcc);
 
 		DBG("[%d] GOT RESPONSE: %s",telesock,msg);
 		return 1;
